#========================================================================================
# package      DE_EPAGES::Amazon::Hooks::Product
# state        public
# description  runs when product is updated.
#========================================================================================
package DE_EPAGES::Amazon::Hooks::Product;

use strict;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( LogError );
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias );
use DE_EPAGES::Amazon::API::Constants
  qw ( %AMAZON_ONLINESTATUS );

#========================================================================================
# function     OnAfterUpdateProduct
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnAfterUpdateProduct( $hParams );
#----------------------------------------------------------------------------------------
# description  OnAfterUpdateProduct is triggered when a product is updated.
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                 <li>Object        - product object       - object
#                 <li>ProductID     - product identifier   - int
#                 <li>Values        - updated values       - ref.hash
#               </ul> | ref.hash.*
#========================================================================================
sub OnAfterUpdateProduct {
    my ($hParams) = @_;

    # only react if stocklevel changed.
    return unless defined $hParams->{'Values'};
    return unless defined $hParams->{'Values'}->{'StockLevel'};

    my $Class = LoadClassByAlias('AmazonOffer');

    my $Offers = $Class->getObjects( 'ByProductID', [ $hParams->{'ProductID'} ] );
    foreach my $Offer (@$Offers){
        my $PortalConfig = $Offer->get('PortalConfig');
        eval {
            if (($Offer->get('OnlineStatus') == $AMAZON_ONLINESTATUS{'OFFER_ONLINE'}) or
                ($Offer->get('OnlineStatus') == $AMAZON_ONLINESTATUS{'OFFER_PROCESSING'})
                ) {
                $PortalConfig->uploadStock([$Offer]) ;
            }

        };
        if (ExistsError()) {
            my $Error = GetError();
            LogError($Error);
        }
    }
}


#========================================================================================
# function     OnDeleteProduct
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnDeleteProduct( $hParams );
#----------------------------------------------------------------------------------------
# description  OnDeleteProduct is triggered when a product is deleted and deletes
#               remaining AmazonOffers
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                 <li>Object        - product object       - object
#                 <li>ProductID     - product identifier   - int
#                 <li>Values        - updated values       - ref.hash
#               </ul> | ref.hash.*
#========================================================================================
sub OnDeleteProduct {
    my ($hParams) = @_;
    my $Class = LoadClassByAlias('AmazonOffer');
    # delete AmazonOffers
    my $Offers = $Class->getObjects( 'ByProductID', [ $hParams->{'ProductID'} ] );
    foreach my $Offer (@$Offers){
        $Offer->delete;
    }

    # update AmazonTransactions
    $Class    = LoadClassByAlias('AmazonTransaction');
    my $aTranslist  = $Class->getObjects('ByProductID', [ $hParams->{'ProductID'} ] );
    foreach my $Trans (@$aTranslist){
        $Trans->set({'Product' => undef });
    }
}

#========================================================================================
# function     OnProductLineItemQuantity
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnProductLineItemQuantity( $hParams );
#----------------------------------------------------------------------------------------
# description  OnProductLineItemQuantity is triggered when we want to
#               disable the available quantity check while importing Amazontransactions
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                 <li>Product                   - product object        - object
#                 <li>LineItem                  - lineitem object       - object
#                 <li>LineItemContainer         - container object      - object
#                 <li>OtherLineItemQuantity     - quantity of same lineitems    - int
#                 <li>Quantity                  - quantity we want to change    - int
#                 <li>DisableQuantityCheck     - flag to disable check - boolean
#               </ul> | ref.hash.*
#========================================================================================
sub OnProductLineItemQuantity {
    my ($hParams) = @_;

    my $Container = $hParams->{'LineItemContainer'};
    my $DisableQuantityCheck = $hParams->{DisableQuantityCheck};

    if ($Container->parent->instanceOf('LineItemAmazonTransaction') && !$DisableQuantityCheck) {
        $hParams->{DisableQuantityCheck} = 1;
    }
}

1;