#==============================================================================
# §package      DE_EPAGES::Amazon::UI::AmazonConfig
# §state        private
#==============================================================================
package DE_EPAGES::Amazon::UI::AmazonConfig;
use base DE_EPAGES::ProductPortal::UI::PortalConfig;

use strict;
use DE_EPAGES::Object::API::Factory qw( LoadClassByAlias );
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::PerlTools qw ( unique );
use DE_EPAGES::Core::API::Warning qw ( BlockWarnings Warning);
use DE_EPAGES::Core::API::Log qw ( LogError LogDebug );
use DE_EPAGES::Amazon::API::Constants qw ( $OAUTH_STATE_HASH_SECRET $OAUTH_TOKEN_ENDPOINT %AMAZON_ITEM_CONDITIONS %AMAZON_ONLINESTATUS REPORT_SCHEDULER_PERIOD );
use DE_EPAGES::Amazon::API::Object::RemoteMerchantToMWSAdapter;
use DE_EPAGES::Amazon::API::RemoteAdapterFactory;
use DE_EPAGES::Search::API::Object::PagerSearch qw ( ReplaceWildcards ReplaceWildcardsWithoutPosOne );
use DE_EPAGES::Search::API::SearchDefinition qw (GetSearchDefinition);
use DE_EPAGES::Shop::API::Url qw ( BuildShopUrl );
use XML::Simple;
use MIME::Base64;
use Digest::SHA qw ( hmac_sha256_base64 );
use JSON;
use DateTime;
use URI;
use URI::Escape qw(uri_escape_utf8);



#========================================================================================
# §function     $self->ViewGeneral
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       ViewGeneral($Servlet);
#----------------------------------------------------------------------------------------
# §description  Subroutine for MBO/UnityMBO View Action MBO-ViewGeneral
#----------------------------------------------------------------------------------------
# §input        $Servlet | Servlet object | object
#========================================================================================
sub ViewGeneral {
    my $self = shift;
    my ($Servlet) = @_;

    my $AmazonConfig = $Servlet->object;
    my $Form = $Servlet->form;

    my $OAuthReturn = $Form->value('OAuthReturn');
    my $Selling_partner_id = $Form->value('selling_partner_id');
    my $Mws_auth_token = $Form->value('mws_auth_token');
    my $Spapi_oauth_code = $Form->value('spapi_oauth_code');

    my $PortalSite = $AmazonConfig->get('PortalSite');
    my $hPortalInfo = $PortalSite->get(['SPAPI_ClientID', 'SPAPI_ClientSecret']);

    if ( $OAuthReturn && $Selling_partner_id && $Mws_auth_token && $Spapi_oauth_code ) {
        my $ClientID = $hPortalInfo->{'SPAPI_ClientID'};
        my $ClientSecret = $hPortalInfo->{'SPAPI_ClientSecret'};

        my $Response = $AmazonConfig->doRequest(
            'POST', 
            $OAUTH_TOKEN_ENDPOINT, 
            "grant_type=authorization_code&code=$Spapi_oauth_code&client_id=$ClientID&client_secret=$ClientSecret", 
            {'Content-Type' => 'application/x-www-form-urlencoded;charset=UTF-8'}, #-- Don't touch me! I'm very fragile and will probably break if you lay a finger on me...
            {}
        );
        if ( $Response && $Response->is_success() ) {
            my $DecodedContent = $Response->decoded_content;

            my $hJsonResponse;
            eval { $hJsonResponse = decode_json($DecodedContent); };
            $hJsonResponse = undef if $@;
            if ( defined $hJsonResponse ) {
                $AmazonConfig->set({
                    'SPAPI_AccessToken' => $hJsonResponse->{'access_token'},
                    'SPAPI_AccessTokenExpiry' => time() + $hJsonResponse->{'expires_in'},
                    'SPAPI_RefreshToken' => $hJsonResponse->{'refresh_token'},
                    'SPAPI_RefreshTokenExpiry' => time() + 31104000, #-- 360 days. We shave 4 days off because well, why not give ourselves a bit more leeway?
                    'MWSMerchantID' => $Selling_partner_id,
                    'MWSAuthToken' => $Mws_auth_token,
                    'OrderCheck'   => 1,
                });
                $Servlet->vars('OAuthSuccess', 1);

                # delete the notification for AmazonAuthExpire if exists
                my $Shop = $AmazonConfig->getSite();
                if ($Shop->folder('ShopNotifications')->existsChild('AmazonAuthExpire')) {
                    my $AmazonAuthExpireNotification = $Shop->folder('ShopNotifications')->child('AmazonAuthExpire');
                    $AmazonAuthExpireNotification->deleteObject($AmazonConfig);
                }

                # Initialise the order report scheduler after successfull auth
                $AmazonConfig->activateOrderScheduler(REPORT_SCHEDULER_PERIOD);

            }
        }
    }

    my $hAmazonConfigInfo = $AmazonConfig->get(['SPAPI_AccessToken', 'SPAPI_AccessTokenExpiry', 'SPAPI_RefreshToken', 'SPAPI_RefreshTokenExpiry']);
    my $SPAPI_AccessToken = $hAmazonConfigInfo->{'SPAPI_AccessToken'};
    my $SPAPI_RefreshToken = $hAmazonConfigInfo->{'SPAPI_RefreshToken'};
    my $SPAPI_RefreshTokenExpiry = $hAmazonConfigInfo->{'SPAPI_RefreshTokenExpiry'};
    if ( $SPAPI_AccessToken && $SPAPI_RefreshToken && $SPAPI_RefreshTokenExpiry ) {
        my $RefreshExpiry = DateTime->from_epoch(
            epoch     => $SPAPI_RefreshTokenExpiry,
        );
        $Servlet->vars('RefreshTokenExpiry', $RefreshExpiry);
    }



    my $ObjectID = $AmazonConfig->id;
    my $ViewAction = 'MBO-ViewGeneral'; #-- Works for both Base or Now, even without UnityMBO for Now

    #-- The redirect URL to pass to Amazon. Amazon redirects to this URL, before we read the 'state' parameter to pass the browser back to the merchant MBO
    #-- Standard OAuth runaround as the redirect url must match one of the redirect urls that is added to Amazon. Since it would be practically impossible to add a merchant specific domain/url
    #-- for every shop in ePages, we add one Site URL per installation. This is comprised of the system domain, Site.admin ext, plus a change action
    #-- redirect_uri=https://docker-alpha.epages.systems/epages/Site.admin/?ChangeAction=BacklinkAmazonOnboarding

    #-- The state parameter holds the final URL we redirect to. Amazon redirects to redirect_uri as above, then we read 'State' to redirect back to the merchant MBO.
    #-- The user is redirected in a browser to the URL as follows:

    #-- A unique domain by country, with the static query string 'apps/authorize/consent?'
    #-- https://sellercentral-europe.amazon.com/apps/authorize/consent?

    #-- The application id that is being authorised
    #-- application_id=amzn1.sellerapps.app.e958e8a7-52a8-4911-8611-48d86e5069e5

    #-- The state parameter, which contains the URL to the portal config General page, with a hash value generated from the host, object id and view action to prevent URL abuse
    #-- This is then base64 encoded for quality of life (because reading URLs within URLs is not easy), and uri encoded

    #-- 1. https://demoger.docker-alpha.epages.systems/epages/demoger.admin/sec54e046b09c/?ViewAction=UnityMBO-ViewGeneral&ObjectID=30689
    #-- 2. uri_escape_utf8( hmac_sha256_base64( "$VerifyHost-$ObjectID-$ViewAction" ) ) ------------------->---------------------->--------------->----------------|
    #-- 3. https://demoger.docker-alpha.epages.systems/epages/demoger.admin/sec54e046b09c/?ViewAction=UnityMBO-ViewGeneral&ObjectID=30689&Hash=XXXXXXXXXXXXXXXXX <-|
    #-- 4. Base64 encode 3., giving us something like aHR0cHM6Ly9kZW1vZ2VyLmRvY2tlci1hbHBoYS5lcGFnZXMuc3lzdGVtcy9lcGFnZXMvZGVtb2dl....bA==
    #--  state=aHR0cHM6Ly9kZW1vZ2VyLmRvY2tlci1hbHBoYS5lcGFnZXMuc3lzdGVtcy9lcGFnZXMvZGVtb2dlci5hZG1pbi9zZWM1NGUwNDZiMDljLz9IYXNoPU1aajEyNThSJTJCVEt4OG9mbngyLy9aMmRvOFF2akVEcVdBQ05iTFFhcjBWYyZPYmplY3RJRD0zMDY4OSZWaWV3QWN0aW9uPU1CTy1WaWV3R2VuZXJhbA==

    #-- Build the state parameter
    my $StateURL = BuildShopUrl( $AmazonConfig, {'ObjectID' => $ObjectID, 'ViewAction' => $ViewAction, 'OAuthReturn' => 1}, {'Type' => 'admin', 'UseSSL' => 1, 'AddToken' => 1}, $Servlet);
    my $VerifyHost = URI->new( $StateURL )->host();

    #-- Base64 only contains A-Z , a-z , 0-9 , + , / and = (don't quote me on that)
    #-- So it needs to be uri escaped because when we parse this value later, characters like +,/,= are removed from query strings, for e.g., because they have a designated purpose in such places
    my $Hash = uri_escape_utf8( hmac_sha256_base64( "$VerifyHost-$ObjectID-$ViewAction", $OAUTH_STATE_HASH_SECRET) );

    #-- Now we add our hash parameter onto the end
    $StateURL = $StateURL . '&Hash=' . $Hash;

    #-- This value does not have to be uri encoded, because it will be used in the browser source, so the browser should escape it for us
    my $State = encode_base64($StateURL);

    $Servlet->vars('State', $State);
}

#==============================================================================
# Event handlers
#==============================================================================

sub RemoveProducts {
    my $self         = shift;
    my ($Servlet)    = @_;

    my $AmazonConfig = $Servlet->object;
    my $Form         = $Servlet->form;

    my $Shop    = $AmazonConfig->getSite();
    my $hValues = $Form->form( $AmazonConfig, 'ListedObjects' );

    my @ListObjects = map { $Shop->loadObject($_->{'ListObjectID'}, 'AmazonOffer') }
                      @{ $hValues->{'ListObjectIDs'} };

    $Servlet->testPermission( $_, 'Save' ) foreach @ListObjects;

    # delete only if accepted or no confirmation is set
    if ( $Servlet->user->get('DeleteConfirmation')
        and not $Servlet->params('ACCEPT') )
    {
        $Servlet->vars( 'DELETE', 1 );
        $Servlet->vars( 'DeleteObjects', [ map { $_->tleHash } @ListObjects ] );
        return;
    }
    else {
        $Servlet->vars( 'DELETE', 0 );
    }

    $AmazonConfig->removeOffers( \@ListObjects );
    return;
}

sub RemoveAllProducts {
    my $self         = shift;
    my ($Servlet)    = @_;

    my $AmazonConfig = $Servlet->object;
    my $aOffers = $AmazonConfig->folder('AmazonOffers')->children;

    # delete only if accepted or no confirmation is set
    if ( $Servlet->user->get('DeleteConfirmation')
        and not $Servlet->params('ACCEPT') )
    {
        $Servlet->vars( 'DELETE', 1 );
        $Servlet->vars( 'DeleteObjects', [ map { $_->tleHash } @{$aOffers} ] );
        return;
    }
    else {
        $Servlet->vars( 'DELETE', 0 );
    }

    $AmazonConfig->removeOffers($aOffers);
    return;
}



sub ExportPortalConfig {
    my $self = shift;
    my ($Servlet) = @_;

    $self->SUPER::ExportPortalConfig( $Servlet, 0 );
    return;
}

sub AddProductsFromTray {
    my $self = shift;
    my ($Servlet) = @_;

    my $aWarnings = BlockWarnings(
        sub {
            eval { $self->SUPER::AddProductsFromTray($Servlet); };
            if ( ExistsError() ) {
                my $Error = GetError();
                LogError($Error) if ( $Error->code =~ /Native/ );
            }
        }
    );

    LogDebug( 'AddProduct Warnings', $aWarnings ) if ( scalar @$aWarnings );

    foreach my $warning (@$aWarnings) {
        if ( $warning->code eq 'NativeWarn' ) {
            Warning($warning);
        }
        else {
            $Servlet->form->mapFormError( {
                    'Form' => 'NewProduct',
                    'Name' => 'Tray'
                },
                $warning
            );
        }
    }

    $Servlet->form->executeFormError;
}

sub AddProduct {
    my $self      = shift;
    my ($Servlet) = @_;
    my $Object    = $Servlet->object;

    my $aWarnings = BlockWarnings(
        sub {
            eval { $self->SUPER::AddProduct( $Servlet, $Object ); };
            if ( ExistsError() ) {
                my $Error = GetError();
                LogError($Error) if ( $Error->code =~ /Native/ );
            }
        }
    );

    LogDebug( 'AddProduct Warnings', $aWarnings ) if ( scalar @$aWarnings );

    foreach my $warning (@$aWarnings) {
        if ( $warning->code eq 'NativeWarn' ) {
            Warning($warning);
        }
        else {
            $Servlet->form->mapFormError( {
                    'Form' => 'NewProduct',
                    'Name' => 'NewProduct'
                },
                $warning
            );
        }
    }

    $Servlet->form->executeFormError;
}

sub EndOffers {
    my $self         = shift;
    my ($Servlet)    = @_;

    my $PortalConfig = $Servlet->object;
    my $Form         = $Servlet->form;
    my $hValues      = $Form->form( $PortalConfig, 'ListedObjects' );
    my $Shop         = $PortalConfig->getSite();

    # Test existance of objects.
    my @ListObjects  = map { $Shop->loadObject($_->{'ListObjectID'}, 'AmazonOffer') }
                       @{ $hValues->{'ListObjectIDs'} };

    $Servlet->testPermission( $_, 'End' ) foreach @ListObjects;

    $PortalConfig->hideOffers( \@ListObjects );
}

sub ViewPagedChildren {
    my $self         = shift;
    my ($Servlet)    = @_;
    my $PortalConfig = $Servlet->object;

    return $self->SearchOffers( $Servlet, $PortalConfig, 1 );
}

sub SearchOffers {
    my $self = shift;
    my ( $Servlet, $IsList ) = @_;
    my $PortalConfig = $Servlet->object;

    $self->CountByStatus( $Servlet, $PortalConfig );

    my $SearchForm = $Servlet->form->form( $PortalConfig, 'SearchProducts' );
    delete $SearchForm->{$_} foreach grep { not defined $SearchForm->{$_} } keys %$SearchForm;
    my $LocalizedAttributeName = LoadClassByAlias('Product')->attribute('Name');
    my $SearchDefinition = GetSearchDefinition( LoadClassByAlias('AmazonOffer'), 'MBO-AmazonConfig' );

    my %Options = (
        'Servlet'          => $Servlet,
        'Object'           => $PortalConfig,
        'SearchDefinition' => $SearchDefinition,
        'Parameters'       => {
            'PortalConfig.ID'  => $PortalConfig->id,
            'SearchLanguageID' => $Servlet->languageID,
            'NameAttributeID'  => $LocalizedAttributeName->id,
        },
        'UriParameters' => {},
    );

    # if we have a ViewPagedChildren, then filter subproducts.
    $Options{'Parameters'}{'IsNotSubProduct'} = 1 if ( defined $IsList );

    foreach my $Parameter ( keys %$SearchForm ) {
        if ( $Parameter eq 'Alias' ) {
            ## UNITY-4143
            #replace '*' with '%' and add % to the end for (Alias/Productnumber)
            $Options{'Parameters'}{$Parameter} = ReplaceWildcardsWithoutPosOne($SearchForm->{$Parameter});
        }
        elsif ( $Parameter eq 'WildSearchString' ) {    #remove '*' at the end by '%' in (Alias/Productnumber)
            my $SearchString = $SearchForm->{$Parameter};
            $SearchString = ReplaceWildcards($SearchString);
            $Options{'Parameters'}{'Alias'} = $SearchString;
        }
        elsif ( $Parameter eq 'OnlineStatus' ) {
            $Options{'Parameters'}{$Parameter} = $SearchForm->{$Parameter};
        }
        $Options{'UriParameters'}{$Parameter} = $SearchForm->{$Parameter};
    }



    my $Search = DE_EPAGES::Search::API::Object::PagerSearch->new( \%Options );
    $Search->statistic;

    # Get all AmazonOffers before BestPriceUpdate because we do want also show AmazonOffers that already have status 99
    # in the warning message (see UNITY-8125)
    my $AmazonOffers = $PortalConfig->folder('AmazonOffers')->children;
    my %hInvalidAmazonOffers = ();
    %hInvalidAmazonOffers = $PortalConfig->updateBestPrice( $Search->items( $Search->get('Page') ) ) if $Search->get('CountOfPages') > 0;

    foreach my $AmazonOffer (@$AmazonOffers) {
        if ($AmazonOffer->get('OnlineStatus') == 99) {
            # Add AmazonOffers to hash, who have already OnlineStatus 99 set
            $hInvalidAmazonOffers{ $AmazonOffer->id} = $AmazonOffer->get('HistoryName');
        }
    }

    if (keys %hInvalidAmazonOffers) {
        $Servlet->vars('InvalidASINsExists', 1);
        $Servlet->vars('InvalidAmazonOffers', \%hInvalidAmazonOffers);
    }

    return $Search->page;
}

sub CountByStatus {
    my $self         = shift;
    my ($Servlet)    = @_;
    my $PortalConfig = $Servlet->object;

    my %hStatusCount = (
        'OnlineStatusCount_0' => 0,
        'OnlineStatusCount_1' => 0,
        'OnlineStatusCount_2' => 0,
        'OnlineStatusCount_4' => 0,
        'OnlineStatusCount_8' => 0,
    );

    map { $hStatusCount{ 'OnlineStatusCount_' . $_->get('OnlineStatus') }++; }
      @{ LoadClassByAlias('AmazonOffer')->getObjects( 'ByPortalConfigID', [ $PortalConfig->id ] ) };

    foreach my $key ( keys %hStatusCount ) {
        $Servlet->vars( $key, $hStatusCount{$key} );
    }
}

sub Save {
    my $self = shift;
    my ($Servlet) = @_;
    # default.
    $self->_saveMWS( $Servlet );

    return;
}

sub _saveMWS {
    my $self         = shift;
    my ($Servlet)    = @_;
    my $Form         = $Servlet->form;
    my $PortalConfig = $Servlet->object;
    my $hFormValues  = $Form->form( $PortalConfig, 'MWSLoginData' );
    # mws does not require contacting amazon support, so we can set ordercheck=1 by default
    $hFormValues->{'OrderCheck'} = 1;
    my $PortalSite   = $PortalConfig->get('PortalSite');

    my $hInfo = $PortalSite->get( [qw( Portal MWSURL MWSKeyID MWSSecretKeyID MWSMarketplaceID SPAPI_EndPoint )] );

    my $Remote = DE_EPAGES::Amazon::API::Object::RemoteMerchantToMWSAdapter->new(
        'AWSURL'         => $hInfo->{'MWSURL'},
        'AWSAccessKeyId' => $hInfo->{'MWSKeyID'},
        'AWSSecretKeyId' => $hInfo->{'MWSSecretKeyID'},
        'Logging'        => $hInfo->{'Portal'}->get('Logging'),
        'MerchantID'     => $hFormValues->{'MWSMerchantID'},
        'MWSAuthToken'   => $hFormValues->{'MWSAuthToken'},
        'MarketplaceID'  => $hInfo->{'MWSMarketplaceID'},
        'SPAPI_EndPoint' => $hInfo->{'SPAPI_EndPoint'},
        'PortalConfig'   => $PortalConfig,
    );
    $self->_activateAccount( $Remote, $PortalConfig, $Form, $hFormValues );
    return;
}

sub _activateAccount {
    my $self = shift;
    my ( $Remote, $PortalConfig, $Form, $hFormValues ) = @_;
    if ( $self->_isLoginDataOk( $Remote, $Form ) ) {
        $PortalConfig->set($hFormValues);
        $self->_activateOrderScheduler( $PortalConfig, $Form );
    }
    return;
}

sub _isLoginDataOk {
    my $self = shift;
    my ( $Remote, $Form ) = @_;
    eval { $Remote->login(); };
    if ( ExistsError() ) {
        my $Error = GetError();
        if ( $Error->code eq 'InvalidMerchantToken' ) {
            $Form->executeFormError( {
                    'Reason' => 'InvalidMerchantToken',
                    'Form'   => 'LoginData',
                    'Name'   => 'PortalMerchantID',
                    'Value'  => 'LoginFailed',
                }
            );
        }
        else {
            $Form->executeFormError( {
                    'Reason' => 'LoginFailed',
                    'Form'   => 'LoginData',
                    'Name'   => 'LoginData',
                    'Value'  => 'LoginFailed',
                }
            );
        }
    }
    else {
        return 1;
    }
}

sub _activateOrderScheduler {
    my $self = shift;
    my ( $PortalConfig, $Form ) = @_;
    eval {
        $PortalConfig->activateOrderScheduler(REPORT_SCHEDULER_PERIOD);
    };
    if ( ExistsError() ) {
        $Form->executeFormError({
            'Reason' => 'LoginMWSFailed',
            'Form'   => 'LoginData',
            'Name'   => 'MWSMerchantID'

        });
    }
}

sub SaveAmazonDefaultSettings {
    my $self = shift;
    my ($Servlet) = @_;

    my $Form         = $Servlet->form;
    my $PortalConfig = $Servlet->object;

    $Form->form( $PortalConfig, 'DefaultValues' );

    $self->SUPER::Save($Servlet);
}

sub UploadToAmazon {
    my $self = shift;
    my ($Servlet) = @_;

    my $PortalConfig = $Servlet->object;
    my $Form         = $Servlet->form;
    my $hValues      = $Form->form( $PortalConfig, 'ListedObjects' );
    my $Shop         = $PortalConfig->getSite();

    # Fetch objects.
    my @ListObjects  = map { $Shop->loadObject($_->{'ListObjectID'}, 'AmazonOffer') }
                       @{ $hValues->{'ListObjectIDs'} };

    my $aUploadOffers;
    my $aEANList = [];
    # filter products without EAN
    foreach my $Offer (@ListObjects) {

        # test if we have a variation master and add all child offers to the list
        if($Offer->hasChildren) {
            my $aSubOffers = $Offer->children();
            push @ListObjects,$_ foreach @$aSubOffers;
            next;
        }

        my $hInfoOffer = $Offer->get(['Product', 'ASIN', 'OnlineStatus']);

        # skip offers that are already online
        next if $hInfoOffer->{'OnlineStatus'}
            ~~ [$AMAZON_ONLINESTATUS{'OFFER_ONLINE'}, $AMAZON_ONLINESTATUS{'OFFER_PROCESSING'}];

        # test if we have an ASIN
        if (defined $hInfoOffer->{'ASIN'}) {
            push @$aUploadOffers, {'Offer' => $Offer };
            next;
        }

        # test if we have an UPCEAN
        my $EAN = $hInfoOffer->{'Product'}->getWithParent('UPCEAN');
        $EAN =~ s/-//g;

        if ($EAN) {
            # add to list for fetching ASIN later
            push @$aUploadOffers, {'Offer' => $Offer, 'EAN' => $EAN };
            push @$aEANList,$EAN;
        } else {
            $Servlet->form->addFormError({
                'Reason' => 'OFFER_PRODUCT_NO_UPCEAN',
                'Name'  => $hInfoOffer->{'Product'}->alias,
                'NameOrAlias' => $hInfoOffer->{'Product'}->get('NameOrAlias'),
                'Product' => $hInfoOffer->{'Product'}->tleHash,
                'ObjectID' => $Offer->id,
            });
        }
    }

    # remove duplicate entries from EAN list
    @$aEANList = unique @$aEANList;

    my $ahProductsForIDs = [];
    if(scalar(@$aEANList)){
        # call MWS for mapping to ASIN
        my $Remote = DE_EPAGES::Amazon::API::RemoteAdapterFactory::NewRemoteMWSProduct( $PortalConfig );
        eval {
            $ahProductsForIDs = $Remote->GetMatchingProductForId( 'EANs' => $aEANList, 'IdType' => 'EAN' );
        };
        if (ExistsError()) {
            my $Error = GetError();
            $Form->executeFormError({
                'Reason' => 'ProductMatchingFailed',
                'Code' => $Error->code,
                'Message' => $Error->{'Vars'}->{'Message'},
                'Name' => $PortalConfig->alias,
            });
        }
    }

    # filter ASINs
    my $ASINMatches;
    foreach my $ResponseElement (@$ahProductsForIDs){

            $ASINMatches->{$ResponseElement->{'Id'}} = [];
            next unless $ResponseElement->{'status'} eq 'Success';
            my $aProducts = $ResponseElement->{'Products'}->{'Product'};

            # we need an array also if we have only one element (JSON problem)
            $aProducts = ref $aProducts eq 'ARRAY' ? $aProducts : [$aProducts];

            foreach my $hProduct (@$aProducts){
                push @{$ASINMatches->{$ResponseElement->{'Id'}}},
                    $hProduct->{'Identifiers'}->{'MarketplaceASIN'}->{'ASIN'};
            }
    }

    foreach my $hOffer (@$aUploadOffers) {

        my $hInfoOffer = $hOffer->{'Offer'}->get(['Product', 'AmazonPrice', 'DeliveryPeriod', 'ASIN']);
        my $HasASIN = 0;

        if ($hInfoOffer->{'ASIN'}) {
            $HasASIN = 1;
        } else {
            # add ASIN if we had none so far
            # test for one or more ASINs
            # only if we have one ASIN we start the offer
            # otherwise we have to do the matching manually
            my $aASINs = $ASINMatches->{$hOffer->{'EAN'}};

            # add form error if we have not only exactly one ASIN match for this offer product
            if (scalar(@$aASINs) != 1) {
                $Servlet->form->addFormError({
                    'Reason'=> scalar(@$aASINs) > 1 ? 'OFFER_PRODUCT_TOO_MANY_ASINS' : 'OFFER_PRODUCT_NO_ASIN',
                    'Name'  => $hOffer->{'Offer'}->get('Product')->alias,
                    'NameOrAlias' => $hOffer->{'Offer'}->get('Product')->get('NameOrAlias'),
                    'Product' => $hInfoOffer->{'Product'}->tleHash,
                    'ObjectID' => $hOffer->{'Offer'}->id,
                });
            } else {
                # set ASIN
                $hOffer->{'Offer'}->set({'ASIN' => $aASINs->[0]});
                $HasASIN = 1;
            }
        }

        my $ExistsPrice          = $hInfoOffer->{'AmazonPrice'} ? 1:0;
        my $ExistsDeliveryPeriod = $hInfoOffer->{'DeliveryPeriod'} ? 1:0;
        my $ExistsCondition      = defined $hInfoOffer->{'Product'}->getWithParent('AmazonCondition') ? 1:0;

        $Servlet->form->addFormError({
            'Reason'=> 'OFFER_PRODUCT_NO_PRICE',
            'Name'  => $hInfoOffer->{'Product'}->alias,
            'NameOrAlias' => $hInfoOffer->{'Product'}->get('NameOrAlias'),
            'Product' => $hInfoOffer->{'Product'}->tleHash,
            'ObjectID' => $hOffer->{'Offer'}->id,
        }) unless $ExistsPrice;

        $Servlet->form->addFormError({
            'Reason'=> 'OFFER_PRODUCT_NO_DELIVERY_PERIOD',
            'Name'  => $hInfoOffer->{'Product'}->alias,
            'NameOrAlias' => $hInfoOffer->{'Product'}->get('NameOrAlias'),
            'Product' => $hInfoOffer->{'Product'}->tleHash,
            'ObjectID' => $hOffer->{'Offer'}->id,
        }) unless $ExistsDeliveryPeriod;

        $Servlet->form->addFormError({
            'Reason'=> 'OFFER_PRODUCT_NO_CONDITION',
            'Name'  => $hInfoOffer->{'Product'}->alias,
            'NameOrAlias' => $hInfoOffer->{'Product'}->get('NameOrAlias'),
            'Product' => $hInfoOffer->{'Product'}->tleHash,
            'ObjectID' => $hOffer->{'Offer'}->id,
        }) unless $ExistsCondition;

        if($HasASIN and $ExistsPrice and $ExistsDeliveryPeriod and $ExistsCondition) {
            # upload offer to Amazon
            $hOffer->{'Offer'}->startOffer();
        }
    }

    $Servlet->form->executeFormError();

    return;
}

1;
