use strict;

use DE_EPAGES::AppStore::API::Object::ImportClient;
use DE_EPAGES::Core::API::EpagesConfig qw( EpagesConfig );
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::Log qw ( GetCommunicationLog LogContext );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( RunOnStore );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );
use DE_EPAGES::Object::API::Language qw ( GetPKeyLanguageByCode );
use Getopt::Long;
use JSON::XS;
use HTTP::Response;
use WWW::Curl::Easy;
use Readonly;

Readonly my $HTTP_TIMEOUT => 180;

sub Main {

    local $| = 1;

    my $Password;
    my $StoreName  = 'Site';
    # Reseller is an unique identifier of epages-installation for mate given by epages for every installation.
    # This value is only used in context of appstore.  Do not confuse it with provider or businessunit or any epages6 business object.
    my $Reseller = EpagesConfig()->get( 'DE_EPAGES::AppStore', 'Reseller' );
    my $FilterByProvider;
    my $Token = EpagesConfig()->get( 'DE_EPAGES::AppStore', 'TokenMATe' );
    my $mateApiUrl = (EpagesConfig()->get( 'DE_EPAGES::AppStore', 'MATeApiUrl' ) // 'https://mate.epages.com/api/') . 'resellers';
    my $Help;
    my $NoOutput;
    my $SleepInSeconds = 0;

    GetOptions(
        'help'           => \$Help,
        'passwd=s'       => \$Password,
        'provider=s'     => \$FilterByProvider,
        'url=s'          => \$mateApiUrl,
        'token=s'        => \$Token,
        'nooutput'       => \$NoOutput,
        'randomsleep=s'  => \$SleepInSeconds,
    );

    usage() if $Help;
    usage('Missing reseller name in epages.conf') unless ($Reseller);

    RunOnStore(
        Store => $StoreName,
        DBPassword => $Password,
        Sub => sub {
            my $Str = JSON->new->utf8->pretty->encode(_exportShopTypes($Reseller, $FilterByProvider));
            if ($mateApiUrl) {
                sleep (int(rand($SleepInSeconds)));
                _sendData($Str, $mateApiUrl, $Token);
            }
            print $Str unless ($NoOutput);
        }
    );
}

sub _sendData {
    my ($Str, $mateApiUrl, $Token) = @_;

    my @headers = (
        'Accept: application/vnd.epages.mate.v1+json',
        'Authorization: Token ' . $Token,
        'Content-Type: application/json',
    );

    my $curl = WWW::Curl::Easy->new;
    $curl->setopt(CURLOPT_TIMEOUT, $HTTP_TIMEOUT);
    $curl->setopt(CURLOPT_URL, $mateApiUrl);
    $curl->setopt(CURLOPT_HEADER, 1);
    $curl->setopt(CURLOPT_HTTPHEADER, \@headers);
    $curl->setopt(CURLOPT_POST, 1);
    $curl->setopt(CURLOPT_POSTFIELDS, $Str);
    $curl->setopt(CURLOPT_NOPROGRESS, 1);
    $curl->setopt(CURLOPT_FOLLOWLOCATION, 1);

    LogContext({ 'source' => 'exportShopTypes' }, sub {

        my $RequestString = "POST $mateApiUrl\n";
        $RequestString .= join("\n", @headers). "\n$Str\n";
        _CommunicationLogger($RequestString);

        my $ResponseString;
        $curl->setopt(CURLOPT_WRITEDATA, \$ResponseString);
        # $ErrorCode might be zero although an error occured
        my $ErrorCode = $curl->perform;
        my $ResponseCode = HTTP::Response->parse($ResponseString)->code;
        _CommunicationLogger($ResponseString);

        if ( $ErrorCode or $ResponseCode !~ /^[1-3]/ ) {
            Error('REQUEST_FAILED', {
                Request     => $RequestString,
                Response    => $ResponseString,
                Status      => $ResponseCode,
                StatusLine  => "Error code: $ErrorCode",
                HTTPTimeout => $HTTP_TIMEOUT,
            });
        }
    });
    return;
}

sub _exportShopTypes {
    my ($Reseller, $FilterByProvider) = @_;

    my $aShopTypeExport = [];
    my $aProviders   = $FilterByProvider
                     ? [LoadRootObject->child('Providers')->child($FilterByProvider)]
                     :  LoadRootObject->child('Providers')->children;
    my $en = GetPKeyLanguageByCode('en');

    foreach my $Provider (@{$aProviders}){
        my $aShopTypes = [];

        foreach my $ShopType (@{$Provider->get('ShopTypes')}){
            push @$aShopTypes, {
                'alias'        => $ShopType->alias,
                'name'         => $ShopType->get('NameOrAlias', $en),
                'baseShopType' => $ShopType->class->existsAttribute('PriceListShopType')
                                ? $ShopType->get('PriceListShopType')
                                : $ShopType->alias,
            };
        }

        push @$aShopTypeExport, {
            'provider'     => $Reseller,
            'businessUnit' => $Provider->alias,
            'shopTypes'    => $aShopTypes
        };
    }

    return $aShopTypeExport;
}

sub _CommunicationLogger {
    my ($InfoToBeLog) = @_;

    my $IsLoggingActivated = EpagesConfig()->get( 'DE_EPAGES::AppStore', 'EnableCommunicationLogOnScript');
    return if ((defined $IsLoggingActivated)&&($IsLoggingActivated eq '0'));

    my $Logger = GetCommunicationLog('AppStore');
    $Logger->debug($InfoToBeLog);

    return;
}

sub usage {
    my ($Message) = @_;

    print STDERR "Error: $Message\n\n" if defined $Message;

    print <<END_USAGE;

Usage:
    perl $0 [options] [flags]

options:
    -passwd         database user password                      (optional)
    -provider       only extract one provider (e.g. epagesdemo) (optional)
    -url            URL to POST data to                         (optional)
    -token          Authorization token                         (optional)
    -randomsleep    max random sleep time in seconds            (optional)

flags:
    -help       show the command line options

Prints ShopTypes of all providers as JSON. Sends data via HTTP POST.

Sample configuration (read from epages.conf):

[DE_EPAGES::AppStore]
TokenMATe=someSecretToken
MATeApiUrl=https://mate.epages.com/api/
Reseller=epagesdemo

Examples:
    perl $0
    perl $0 -url https://mate.epages.com/api/resellers
END_USAGE
    exit 2;
}

RunScript(
    'Sub' => \&Main
);
