use strict;
use Getopt::Long;
use Time::HiRes qw (time);
use DE_EPAGES::Core::API::Error qw( GetError ExistsError );
use DE_EPAGES::Object::API::Factory qw( LoadRootObject );
use DE_EPAGES::Core::API::Log qw( LogError );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( RunOnStore );
use DE_EPAGES::WebInterface::API::MessageCenter qw( SynchronizeCache );
use DE_EPAGES::Product::API::Shop qw( UpdateAllProductsImageInformation ); ## no critic qw(RequireCorrectImports)

#========================================================================================
# function         UpdateImageInformations
# state            public
#----------------------------------------------------------------------------------------
# syntax           UpdateImageInformations();
#----------------------------------------------------------------------------------------
# description      Fills the ImageInformations attribute on products for a Unity shop
#========================================================================================
sub UpdateImageInformations {
    my ($ShopAlias, $NoOutput) = @_;

    my $ShopsObject = LoadRootObject()->child('Shops');
    my $aShops;
    if ( $ShopAlias ) {
        printf "Reading Unity shop '%s'\n\n", $ShopAlias unless $NoOutput;
        push @$aShops, $ShopsObject->child($ShopAlias);
    } else {
        print "Reading all Unity shops\n\n" unless $NoOutput;
        $aShops = $ShopsObject->children();
    }

    foreach my $Shop (@$aShops) {
        next if ! $Shop->get('IsUnityRegistered');

        # update ContentObject Product (all products and sub products) on shop
        printf "update image informations on shop %s\n", $Shop->id unless $NoOutput;

        eval {
            my $NumUpdated = UpdateAllProductsImageInformation( $Shop );
            printf "  updated %d products\n", $NumUpdated unless $NoOutput;       
        };
        warn('  Update image informations failed for shop ID ' . $Shop->id . " with error \"" . GetError() . "\".") if ExistsError();
    }

    eval { SynchronizeCache(); };
    LogError(GetError()) if ExistsError();

    return;
}

sub Main {

    local $| = 1;

    my $Password;
    my $StoreName;
    my $ShopAlias;
    my $Help;
    my $NoOutput;

    GetOptions(
        'help'        => \$Help,
        'passwd=s'    => \$Password,
        'storename=s' => \$StoreName,
        'shop=s'      => \$ShopAlias,
        'nooutput'    => \$NoOutput,
    );

    usage() if $Help;

    unless ( defined $StoreName ) {
        print STDERR "Missing parameter -storename.\n\n";
        usage();
    }
    my $time1 = time;

    RunOnStore(
        Store      => $StoreName,
        DBPassword => $Password,
        Sub        => sub { UpdateImageInformations($ShopAlias, $NoOutput); },
    );
    my $time2 = time;
    printf "need %.1f seconds\n", $time2 - $time1;
}

sub usage {
    print <<END_USAGE;
Description:
    Fills the ImageInformations attribute on products for a Unity shop.

Usage:
    perl $0 [options] [flags] datafiles

options:
    -passwd     database user password                          ()
    -storename  name of store                                   ()
    -shop       shop alias                                      ()
flags:
    -help       show the command line options
    -nooutput   do not print any output

Example:
    perl $0 -storename Store -shop DemoUnity
END_USAGE
    exit 2;
}

RunScript( 'Sub' => \&Main );