#========================================================================================
# package      DE_EPAGES::DHLIntraship::API::SOAP::BuildTransportLayer
# state        public
#----------------------------------------------------------------------------------------
# description  build the transport layer for Soap request
#----------------------------------------------------------------------------------------
# example     my $TransportLayerBuilder = 
#               DE_EPAGES::DHLIntraship::API::SOAP::BuildTransportLayer->new(
#                   $self->{'OperationName'},
#                   $self->{'hLoginData'}
#                   )
#                   ->withSSLPort();
#               my TransportLayer = $TransportLayerBuilder->getTransportLayer();
#========================================================================================
package DE_EPAGES::DHLIntraship::API::SOAP::BuildTransportLayer;

use strict;
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::DHLIntraship::API::Constants qw ( %SOAP_REQUEST_PORT );
use XML::Compile::Transport::SOAPHTTP;
use LWP::UserAgent;
use URI;


#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new( $OperationName,$hLoginData );
#----------------------------------------------------------------------------------------
# description  constructor
#----------------------------------------------------------------------------------------
# input        $OperationName | soap operation name | String
# input        $hLoginData | login data | ref_hash
# return       $self | self | DE_EPAGES::DHLIntraship::API::SOAP::BuildTransportLayer
#========================================================================================
sub new {
    my $class = shift;
    my ($OperationName,$hLoginData)  = @_;
    my $self  = bless {
        'OperationName'=>$OperationName,
    }, $class;
    $self->_setLoginData($hLoginData);
    return $self;
}

#========================================================================================
# function     _setLoginData
# state        private
#----------------------------------------------------------------------------------------
# syntax       _setLoginData( $hLoginData );
#----------------------------------------------------------------------------------------
# description  set the logindata to class fields
#----------------------------------------------------------------------------------------
# input        $hLoginData | login data | ref_hash
#========================================================================================
sub _setLoginData {
    my $self = shift;
    my ($hLoginData) = @_;
    $self->{'Realm'} = $hLoginData->{'Realm'};
    $self->{'UserName'} = $hLoginData->{'UserName'};
    $self->{'Password'} = $hLoginData->{'Password'};
    $self->{'EndPointURL'} = $hLoginData->{'EndPointURL'};
    return;
}

#========================================================================================
# function     getTransportLayer
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->_getTransportLayer( );
#----------------------------------------------------------------------------------------
# description  prepares the http transportlayer for SOAP request
#----------------------------------------------------------------------------------------
# return       $HTTPTransportLayer | HTTP transport layer | XML::Compile::Transport::SOAPHTTP
#========================================================================================
sub getTransportLayer {
    my $self  = shift;
    my $HTTPTransportLayer = XML::Compile::Transport::SOAPHTTP->new(
        user_agent => $self->_getLWPUserAgent( ),
        keep_alive => 1,
        timeout    => 30,
        address    => $self->{'EndPointURL'},
    );
    return $HTTPTransportLayer;
}

#========================================================================================
# function     _getLWPUserAgent
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_getLWPUserAgent( );
#----------------------------------------------------------------------------------------
# description  preparing the user agent for the ssl-soap request
#----------------------------------------------------------------------------------------
# return       $UserAgent | user agent | LWP::UserAgent
#========================================================================================
sub _getLWPUserAgent {
    my $self = shift;

    if( not $self->{'EndPointURL'} =~ /^http/) {
        Error('EndPointURL has to begin with protocol (http or https)');
    }

    my $HostName = URI->new($self->{'EndPointURL'},'http')->host();
    my $Host = sprintf('%s:%u', $HostName, $self->{'PortNumber'});

    my $UserAgent = LWP::UserAgent->new();
    $UserAgent->credentials(
        $Host,
        $self->{'Realm'},
        $self->{'UserName'},
        $self->{'Password'}
    );

    my $SoapAction = '"urn:' . $self->{'OperationName'} . '"';

    $UserAgent->default_header('SOAPAction' => $SoapAction);

    return $UserAgent;
}

#========================================================================================
# function     withPort
# state        public
#----------------------------------------------------------------------------------------
# syntax       withPort( $PortNumber );
#----------------------------------------------------------------------------------------
# description  set the port number
#----------------------------------------------------------------------------------------
# input        $PortNumber |  port number | interger
# return       $self | self for call chaining | DE_EPAGES::DHLIntraship::API::SOAP::BuildTransportLayer
#========================================================================================
sub withPort {
    my $self = shift;
    my ($PortNumber) = @_;
    if(!isValid($PortNumber)) {
        $PortNumber = $SOAP_REQUEST_PORT{'NO_SSL'};
    }
    $self->{'PortNumber'} = $PortNumber;
    return $self;
}

#========================================================================================
# function     withSSLPort
# state        public
#----------------------------------------------------------------------------------------
# syntax       withSSLPort( $PortNumber );
#----------------------------------------------------------------------------------------
# description  set the ssl port number
#----------------------------------------------------------------------------------------
# input        $PortNumber |  port number | interger
# return       $self | self for call chaining | DE_EPAGES::DHLIntraship::API::SOAP::BuildTransportLayer
#========================================================================================
sub withSSLPort {
    my $self = shift;
    my ($PortNumber) = @_;
    if(!isValid($PortNumber)) {
        $PortNumber = $SOAP_REQUEST_PORT{'SSL'};
    }
    $self->withPort($PortNumber);
    return $self;
}

1;