#========================================================================================
# package      DE_EPAGES::DHLIntraship::API::SOAP::BuildWSDL11Client
# state        public
#----------------------------------------------------------------------------------------
# description  encapsulate the generation of the extended WSDL11 Client
#----------------------------------------------------------------------------------------
# example      my $WSDL11Client = DE_EPAGES::DHLIntraship::API::SOAP::BuildWSDL11Client->new($self->{'ShippmentDD'})
#                                   ->withTransportLayer($TransportLayer)
#                                   ->withPortName('SOAP11Portname');
#========================================================================================
package DE_EPAGES::DHLIntraship::API::SOAP::BuildWSDL11Client;
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::DHLIntraship::API::Constants qw ( @SERVICES @PORTS );
use URI;
use XML::Compile::WSDL11;

use strict;

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new( $OperationName );
#----------------------------------------------------------------------------------------
# description  constructor
#----------------------------------------------------------------------------------------
# input        $OperationName | name of SOAP operation | string
# input        $IsTest | wether is test or not | boolean
#----------------------------------------------------------------------------------------
# return       $CompiledClient | self| DE_EPAGES::DHLIntraship::API::SOAP::BuildWSDL11Client
#========================================================================================
sub new {
    my $class           = shift;
    my ( $OperationName, $IsTest ) = @_;

    my $self  = bless {
            'aXsdFilePaths' => undef,
            'IsTest' => $IsTest,
            'OperationName' => $OperationName,
            'ServiceName' => undef,
            'TransportLayer' => undef,
            'WSDLPath' => undef,
    }, $class;

    $self->withServiceName();
    $self->withPortName();

    return $self;
}

#========================================================================================
# function     compile
# state        private
#----------------------------------------------------------------------------------------
# syntax       compile( );
#----------------------------------------------------------------------------------------
# description  assemble and compile the WSDL11 Client
#----------------------------------------------------------------------------------------
# return       $CompiledClient | the compliled WSDL11 client | object
#========================================================================================
sub compile {
    my $self = shift;

    my $CompiledClient = $self->_getWSDL()->compileClient(
        'operation' => $self->{'OperationName'},
        'service'   => $self->{'ServiceName'},
        'port'      => $self->{'PortName'},
        'transport' => $self->{'TransportLayer'},
    );
    return $CompiledClient;
}

#========================================================================================
# function     _getWSDL
# state        private
#----------------------------------------------------------------------------------------
# syntax       _getWSDL( );
#----------------------------------------------------------------------------------------
# description  loads wsdl and xsd files, and generate the WSDL11 object
#----------------------------------------------------------------------------------------
# return       $WSDL | prepared WSDL11 object | object
#========================================================================================
sub _getWSDL {
    my $self = shift;

    my $WSDL = '';
    if ( isValid($self->{'WSDLPath'})){
        $WSDL = XML::Compile::WSDL11->new($self->{'WSDLPath'});
        $WSDL = $self->_loadXSDFiles($WSDL);
    }else{
        Error('To use the WSDLClient you need a *.wsdl file ( use withWSDLFile("path/to/file"); )');
    }

    return $WSDL;
}
#========================================================================================
# function     _loadXSDFiles
# state        private
#----------------------------------------------------------------------------------------
# syntax       _loadXSDFiles( $WSDL11Object );
#----------------------------------------------------------------------------------------
# description  loads xsd files
#----------------------------------------------------------------------------------------
# input        $WSDL11Object | wsdl11 object | XML::Compile::WSDL11
# return       $WSDL11Object | wsdl11 object | XML::Compile::WSDL11
#========================================================================================
sub _loadXSDFiles {
    my $self = shift;
    my ($WSDL11Object) = @_;

    if (isValid($self->{'aXsdFilePaths'})){
        foreach my $XsdFilePath (@{$self->{'aXsdFilePaths'}}){
            $WSDL11Object->importDefinitions($XsdFilePath);
        }
    }

    return $WSDL11Object;
}

#========================================================================================
# function     withXSDFiles
# state        public
#----------------------------------------------------------------------------------------
# syntax       withXSDFiles( $aXSDFiles );
#----------------------------------------------------------------------------------------
# description  stores the path to the xsd files into the self object
#----------------------------------------------------------------------------------------
# input        $aXSDFiles | path to the xsd files | refarray
# return       $self | self for call chaining | object
#========================================================================================
sub withXSDFiles {
    my $self = shift;
    my ($aXSDFiles) = @_;
    $self->{'aXsdFilePaths'} = $aXSDFiles;
    return $self;
}

#========================================================================================
# function     withWSDLFile
# state        public
#----------------------------------------------------------------------------------------
# syntax       withWSDLFile( $WSDLPath );
#----------------------------------------------------------------------------------------
# description  stores the path to the wsdl file into the self object
#----------------------------------------------------------------------------------------
# input        $WSDLPath | path to the wsdl file | string
# return       $self | self for call chaining | object
#========================================================================================
sub withWSDLFile {
    my $self = shift;
    my ($WSDLPath) = @_;
    $self->{'WSDLPath'} = $WSDLPath;
    return $self;
}

#========================================================================================
# function     withServiceName
# state        public
#----------------------------------------------------------------------------------------
# syntax       withServiceName( );
#----------------------------------------------------------------------------------------
# description  set the service name
#========================================================================================
sub withServiceName {
    my $self = shift;

    $self->{'ServiceName'} = $SERVICES[0];

    return;
}

#========================================================================================
# function     withPortName
# state        public
#----------------------------------------------------------------------------------------
# syntax       withPortName( );
#----------------------------------------------------------------------------------------
# description  set the port name
#========================================================================================
sub withPortName {
    my $self = shift;

    $self->{'PortName'} = $PORTS[0];

    return;
}

#========================================================================================
# function     withTransportLayer
# state        public
#----------------------------------------------------------------------------------------
# syntax       withTransportLayer( TransportLayer );
#----------------------------------------------------------------------------------------
# description  set the TransportLayer
#----------------------------------------------------------------------------------------
# input        $TransportLayer | TransportHook Object| HTTP::Response
# return       $self | self for call chaining | object
#========================================================================================
sub withTransportLayer {
    my $self = shift;
    my ($TransportLayer) = @_;
    $self->{'TransportLayer'} = $TransportLayer;
    return $self;
}

1;