#========================================================================================
# package      DE_EPAGES::DHLIntraship::t::unit::BuildWSDL11Client
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test of BuildWSDL11Client
#========================================================================================
package DE_EPAGES::DHLIntraship::t::unit::BuildWSDL11Client;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::DHLIntraship::API::Constants qw (
    $DHLINTRASHIP_PATH_WSDL
    $LOCATION_WSDL
    @PORTS
    @SERVICES
    $SHIPPING_WSDL
    $XSD_CISBASE_FILE
    $XSD_ISBASE_FILE
    $XSD_STANDORTE_FILE
);
use DE_EPAGES::DHLIntraship::API::SOAP::BuildWSDL11Client;

use LWP::UserAgent;
use XML::Compile::SOAP11;
use XML::Compile::Transport::SOAPHTTP;

use Test::Exception;
use Test::More;

use strict;

use constant CREATE_SHIPMENT_ORDER_OPERATION => 'createShipmentOrder';
use constant GET_MANIFEST_OPERATION => 'getManifest';
use constant IS_TEST => 1;
use constant TEST_LOGIN_DATA => {
        'UserName' => 'TestUser',
        'Password' => 'TestPassword',
        'Realm' => 'TestRealm',
        'EndPointURL' => 'https://www.test.example',
};
use constant WSDL_XSD_FILES_PATH => $ENV{'EPAGES_CARTRIDGES'} . $DHLINTRASHIP_PATH_WSDL;

sub test {
    my $self = shift;

    $self->Test_BuildWSDL11Client_with_createShipmentOrder_operation();
    $self->Test_BuildWSDL11Client_with_getManifest_operation();
    $self->Test_ErrorHandlingIfWSDLFileIsNotSet();
    done_testing();
}

sub Test_BuildWSDL11Client_with_createShipmentOrder_operation {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $WSDL11Client = DE_EPAGES::DHLIntraship::API::SOAP::BuildWSDL11Client->new(
            CREATE_SHIPMENT_ORDER_OPERATION, IS_TEST
    );
    is($WSDL11Client->{'ServiceName'}, $SERVICES[0], "$SubTestName - Service name has been properly set up.");
    is($WSDL11Client->{'PortName'}, $PORTS[0], "$SubTestName - Port name has been properly set up.");

    my $HttpTransport = $self->_createTransport(CREATE_SHIPMENT_ORDER_OPERATION);
    $WSDL11Client->withTransportLayer($HttpTransport);
    isa_ok($WSDL11Client->{'TransportLayer'},
            'XML::Compile::Transport::SOAPHTTP', "$SubTestName - Transport layer set it up.");

    my $WSDLShippingFile = WSDL_XSD_FILES_PATH . $SHIPPING_WSDL;
    $WSDL11Client->withWSDLFile($WSDLShippingFile);
    like($WSDL11Client->{'WSDLPath'}, qr#$DHLINTRASHIP_PATH_WSDL#, "$SubTestName - WSDL file ready to be imported into WSDL11 object.");

    my $aXSDShippingFiles = [ WSDL_XSD_FILES_PATH . $XSD_CISBASE_FILE, WSDL_XSD_FILES_PATH . $XSD_ISBASE_FILE ];
    $WSDL11Client->withXSDFiles($aXSDShippingFiles);
    like($WSDL11Client->{'aXsdFilePaths'}->[0], qr#$XSD_CISBASE_FILE#, "$SubTestName - XSD CISBASE file ready to be imported into WSDL11 object.");
    like($WSDL11Client->{'aXsdFilePaths'}->[1], qr#$XSD_ISBASE_FILE#, "$SubTestName - XSD ISBASE file ready to be imported into WSDL11 object.");

    my $CompiledClient = $WSDL11Client->compile();
    ok(defined($CompiledClient), "$SubTestName - Get compiled WSDL11 client.");
}

sub Test_BuildWSDL11Client_with_getManifest_operation {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $WSDL11Client = DE_EPAGES::DHLIntraship::API::SOAP::BuildWSDL11Client->new(
            GET_MANIFEST_OPERATION, IS_TEST
    );
    is($WSDL11Client->{'ServiceName'}, $SERVICES[0], "$SubTestName - Service name has been properly set up.");
    is($WSDL11Client->{'PortName'}, $PORTS[0], "$SubTestName - Port name has been properly set up.");
}


sub Test_ErrorHandlingIfWSDLFileIsNotSet {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $WSDL11Client = DE_EPAGES::DHLIntraship::API::SOAP::BuildWSDL11Client->new();

    throws_ok {$WSDL11Client->compile() } qr/To use the WSDLClient/,
              "$SubTestName - test error handling if the wsdl file is not set - an Error message";
}

#----------------------------------------------------------------------------------------
# Helpers
#----------------------------------------------------------------------------------------
sub _createTransport {
    my $self = shift;
    my ($SoapAction) = @_;

    my $HttpTransport = XML::Compile::Transport::SOAPHTTP->new(
        user_agent => $self->_createUserAgent($SoapAction),
        keep_alive => 1,
        timeout    => 30,
        address    => TEST_LOGIN_DATA->{'EndPointURL'},
    );

    return $HttpTransport;
}

sub _createUserAgent {
    my $self = shift;
    my ($SoapAction) = @_;

    my $UserAgent = LWP::UserAgent->new();
    $UserAgent->credentials(
        TEST_LOGIN_DATA->{'EndPointURL'},
        TEST_LOGIN_DATA->{'Realm'},
        TEST_LOGIN_DATA->{'UserName'},
        TEST_LOGIN_DATA->{'Password'}
    );
    $UserAgent->default_header('SOAPAction' => $SoapAction);

    return $UserAgent;
}

#----------------------------------------------------------------------------------------
# Run Script
#----------------------------------------------------------------------------------------
__PACKAGE__->runScript();