package DE_EPAGES::GoogleSmartShopping::API::GooglePortalService;

use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );

use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Log qw( LogError );
use DE_EPAGES::GoogleSmartShopping::API::Tools qw ( LogGoogleSmartShopping );

use strict;

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::GoogleSmartShopping::API::GooglePortalService->new( %args )
#----------------------------------------------------------------------------------------
# description  Creates new GooglePortalServiceObject
#----------------------------------------------------------------------------------------
# input        $hSettings | hashref | ShopObject | object
#========================================================================================

sub new {
    my $class = shift;
    my ($hSettings)  = @_;

    my $self  = bless {
        Shop => $hSettings->{'Shop'},
    }, $class;

    return $self;
}

#========================================================================================
# function     getSmartShoppingPortalConfig
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->getSmartShoppingPortalConfig( );
#----------------------------------------------------------------------------------------
# description  Returns GoogleSmartShoppingConfig from Shop
#----------------------------------------------------------------------------------------
# return       PortalConfig Object | Object
#========================================================================================

sub getSmartShoppingPortalConfig {
    my $self = shift;

    my $PortalConfigs = $self->{'Shop'}->get('PortalConfigs');
    return unless defined $PortalConfigs;

    foreach my $PortalConfig (@{$PortalConfigs}){
        if (index($PortalConfig->get('Alias') ,'GoogleSmartShopping') != -1) {

            return $PortalConfig;
        }
    }

    return;
}

#========================================================================================
# function     getPortalConfigCreationParams
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->getPortalConfigCreationParams( );
#----------------------------------------------------------------------------------------
# description  Returns PortalConfig Creation Parameters
#----------------------------------------------------------------------------------------
# return       $hashRef
#========================================================================================

sub getPortalConfigCreationParams {
    my $self = shift;

    my $GoogleSmartShoppingPortal = $self->_getGoogleSmartShoppingPortal();

    unless (defined $GoogleSmartShoppingPortal) {
        LogGoogleSmartShopping("getPortalConfigCreationParams failed - GoogleSmartShoppingPortal Not Found", {});
        return;
    }
    my $LocalizedPortalSite = $self->_getLocalizedPortalSite($GoogleSmartShoppingPortal);

    unless (defined $LocalizedPortalSite) {
        LogGoogleSmartShopping("getPortalConfigCreationParams failed - LocalizedPortalSite Not Found", {});
        return;
    }

    my $PortalConfigFolder = $self->{'Shop'}->folder('PortalConfigs');

    unless (defined $PortalConfigFolder) {
        LogGoogleSmartShopping("getPortalConfigCreationParams failed - PortalConfigFolder Not Found", {});
        return;
    }

    my $Class = $GoogleSmartShoppingPortal->get('PortalConfigClass');

    return {
        'Class'      => $Class,
        'Folder'     => $PortalConfigFolder,
        'PortalSite' => $LocalizedPortalSite
    }
}

sub _getGoogleSmartShoppingPortal {
    my $self = shift;

    my $System = LoadRootObject();
    my $aPortals = $System->get('Portals');

    foreach my $Portal (@{$aPortals}) {
        if ($Portal->get('NameOrAlias') eq 'GoogleSmartShopping') {

            return $Portal;
        }
    }

    return;
}

sub _getLocalizedPortalSite {
    my $self = shift;
    my ($GoogleSmartShoppingPortal) = @_;

    my $ShopLocaleID = $self->{'Shop'}->get('LocaleID');
    foreach my $LocalizedPortalSite (@{$GoogleSmartShoppingPortal->get('PortalSites')}) {
        if ($LocalizedPortalSite->get('LocaleID') eq $ShopLocaleID) {

            return $LocalizedPortalSite;
        }
    }

    LogError("Could not find PortalSitesGoogleSmartShopping.xml with Locale: $ShopLocaleID");
    return;
}

#========================================================================================
# function     deleteSmartShoppingPortalConfig
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->deleteSmartShoppingPortalConfig( );
#----------------------------------------------------------------------------------------
# description  deletes GoogleSmartshopping Config
#========================================================================================

sub deleteSmartShoppingPortalConfig {
    my $self = shift;
    my $SmartShoppingPortalConfig = $self->getSmartShoppingPortalConfig();
    return unless defined $SmartShoppingPortalConfig;

    $SmartShoppingPortalConfig->delete();
}

#========================================================================================
# function     deleteProductsInPortalConfig
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->deleteProductsInPortalConfig($aProductAliases);
#----------------------------------------------------------------------------------------
# description  deletes products in GoogleSmartshopping Config
#----------------------------------------------------------------------------------------
# input        $aProductAliases | Array
#========================================================================================

sub deleteProductsInPortalConfig {
    my $self = shift;
    my($aProductAliases) = @_;

    my $aProductsInShop = [];
    my $aProductALiasesNotInShop = [];
    foreach my $ProductAlias (@{$aProductAliases}) {
        if ($self->{'Shop'}->folder('Products')->existsChild($ProductAlias)) {
            my $ProductByAlias = $self->{'Shop'}->folder('Products')->child($ProductAlias);
            push (@$aProductsInShop, $ProductByAlias);
        }
        else {
            push(@$aProductALiasesNotInShop, $ProductAlias);
        }
    }

    my $PortalConfig = $self->getSmartShoppingPortalConfig();
    $PortalConfig->removeProducts($aProductsInShop);
    if (scalar @$aProductAliases > scalar @$aProductsInShop) {
        LogGoogleSmartShopping('deleteProductsInPortalConfig: Array contained following aliases not in Shop:', {
            'ProductAliasesNotInShop' => $aProductALiasesNotInShop,
        });
    }

    return;
}

#========================================================================================
# function     getProductAliasByGoogleID
# state        public
#----------------------------------------------------------------------------------------
# syntax       g$self->etProductAliasByGoogleID($aProductGoogleIDs);
#----------------------------------------------------------------------------------------
# description  returns Product Aliases from GoogleProductIDs
#----------------------------------------------------------------------------------------
# input        $aProductGoogleIDs | ArrayRef |
# return       $aProductAliases | Array |
#========================================================================================

sub getProductAliasByGoogleID {
    my $self = shift;
    my ( $aProductGoogleIDs) = @_;
    my $aProductAliases = [];
    foreach my $ProductGoogleID (@{$aProductGoogleIDs}) {
        eval {
            my @OfferParts = split(/:/, $ProductGoogleID);
            my $AliasWithLang = pop @OfferParts;
            my @AliasWithLangParts = split(/_/, $AliasWithLang);
            push(@$aProductAliases, $AliasWithLangParts[0]);

        };
        if (ExistsError()) {
            my $Error = GetError();
            LogGoogleSmartShopping('_getProductAliasByGoogleID -Error parsing externalProductId ', { 'Error' => $Error->message() });
        }
    }

    return $aProductAliases;
}
1;
