#========================================================================================
# package      DE_EPAGES::GoogleSmartShopping::t::unit::GooglePortalService
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for API::GooglePortalService
#========================================================================================
package DE_EPAGES::GoogleSmartShopping::t::unit::GooglePortalService;
use base DE_EPAGES::Core::API::Test;


use DE_EPAGES::Test::Mock::MockifyEpagesObject;

use Test::Mockify::Sut;
use Test::Exception;
use Test::MockObject::Extends;
use Test::Mockify::Matcher qw ( String );
use Test::Mockify::Verify qw ( WasCalled );
use Test::More;

use DE_EPAGES::Object::API::Factory;
use DE_EPAGES::Object::API::Object::Object;

use strict;

use constant GOOGLE_SMART_SHOPPING_PORTAL_ALIAS => 'GoogleSmartShopping';
use constant LOCALE_ID => 'en_GB';
use constant PORTAL_CONFIG_FOLDER => 'PortalConfigs';
use constant PORTAL_CONFIG_CLASS => 'SomePortalConfigClass';
use constant DO_NOT_RETURN_FOLDER => 1;
sub test {
    my $self = shift;

    $self->test_getGoogleSmartShoppingPortal_success();
    $self->test_getGoogleSmartShoppingPortal_fail();

    $self->test_GetPortalConfigParams_success();
    $self->test_GetPortalConfigParams_fail_no_portals();
    $self->test_GetPortalConfigParams_fail_no_portal_sites();
    $self->test_DeleteGooglePortalConfig_success();
    $self->test_DeleteProductsInGooglePortalConfig_success();
    $self->test_getProductAliasByGoogleID_success();

    done_testing();
}

sub test_getGoogleSmartShoppingPortal_success {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];
    
    my $MockedGoogleSmartShoppingPortalConfig = $self->_createGoogleSmartShoppingPortalConfig();
    my $MockedIdealoPortalConfig = $self->_createIdealoConfig();
    
    my $MockedShop = $self->_createShop({'PortalConfigs' => [$MockedGoogleSmartShoppingPortalConfig, $MockedIdealoPortalConfig] });

    my $SUT = $self->_createSUT($MockedShop);

    my $GetSmartShoppingPortalConfigResponse = undef;

    lives_ok(
        sub {
            $GetSmartShoppingPortalConfigResponse = $SUT->getSmartShoppingPortalConfig();
        },
        "$SubTestName - getSmartShoppingPortalConfig can be called without errors."
    );

    is( $GetSmartShoppingPortalConfigResponse->get('Alias'), GOOGLE_SMART_SHOPPING_PORTAL_ALIAS, "$SubTestName - A Portal with the Alias GoogleSmartShopping is returned.");
}

sub test_getGoogleSmartShoppingPortal_fail {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedIdealoPortalConfig = $self->_createIdealoConfig();

    my $MockedShop = $self->_createShop( {'PortalConfigs' => [$MockedIdealoPortalConfig] });

    my $SUT = $self->_createSUT($MockedShop);
    my $GetSmartShoppingPortalConfigResponse = undef;

    lives_ok(
        sub {
            $GetSmartShoppingPortalConfigResponse = $SUT->getSmartShoppingPortalConfig();
        },
        "$SubTestName - getSmartShoppingPortalConfig can be called without errors."
    );

    is( $GetSmartShoppingPortalConfigResponse, undef, "$SubTestName - No Portal Is returned.");

}

sub test_GetPortalConfigParams_success {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedShop = $self->_createShop();

    my $GoogleSmartShoppingPortal = $self->_createPortal({ 'NameOrAlias' => GOOGLE_SMART_SHOPPING_PORTAL_ALIAS });
    my $IdealoPortal = $self->_createPortal({ 'NameOrAlias' => 'Idealo' });

    my $SUT = $self->_createSUT($MockedShop,[$GoogleSmartShoppingPortal, $IdealoPortal]);

    my $PortalConfigCreationParams = undef;
    lives_ok(
        sub {
            $PortalConfigCreationParams = $SUT->getPortalConfigCreationParams();
        },
        "$SubTestName - GetPortalConfigCreationParams can be called without errors."
    );

    is($PortalConfigCreationParams->{'Folder'}->get('Alias'), PORTAL_CONFIG_FOLDER, "$SubTestName - Portal Config Folder is returned correctly." );
    is($PortalConfigCreationParams->{'Class'}, PORTAL_CONFIG_CLASS, "$SubTestName - Portal Config Class is returned correctly." );
    is($PortalConfigCreationParams->{'PortalSite'}->get('LocaleID'), LOCALE_ID, "$SubTestName - PortalSite has Correct LocaleID");
}

sub test_GetPortalConfigParams_fail_no_portals {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedShop = $self->_createShop();
    my $IdealoPortal = $self->_createPortal({ 'NameOrAlias' => 'Idealo' });

    my $SUT = $self->_createSUT($MockedShop,[$IdealoPortal]);

    my $PortalConfigCreationParams = undef;
    lives_ok(
        sub {
            $PortalConfigCreationParams = $SUT->getPortalConfigCreationParams();
        },
        "$SubTestName - GetPortalConfigCreationParams can be called without errors."
    );
    is($PortalConfigCreationParams, undef, "$SubTestName - GetPortalConfigCreationParams has returned early, because there is no GoogleSmartShoppingPortal in our System")

}

sub test_GetPortalConfigParams_fail_no_portal_sites {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedShop = $self->_createShop();
    my $GoogleSmartShoppingPortal = $self->_createPortal({'NameOrAlias' => GOOGLE_SMART_SHOPPING_PORTAL_ALIAS, 'PortalSiteLocaleID' => 'de_CH' });
    my $IdealoPortal = $self->_createIdealoConfig();

    my $SUT = $self->_createSUT($MockedShop,[$GoogleSmartShoppingPortal, $IdealoPortal]);

    my $PortalConfigCreationParams = undef;
    lives_ok(
        sub {
            $PortalConfigCreationParams = $SUT->getPortalConfigCreationParams();
        },
        "$SubTestName - GetPortalConfigCreationParams can be called without errors."
    );

    is($PortalConfigCreationParams, undef, "$SubTestName - GetPortalConfigCreationParams has returned early, because there are no PortalSites with matching LocaleID")

}

sub test_DeleteGooglePortalConfig_success {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingPortal = $self->_createPortal({'NameOrAlias' => GOOGLE_SMART_SHOPPING_PORTAL_ALIAS, 'PortalSiteLocaleID' => 'de_CH' });
    my $GooglePortalConfig = $self->_createGoogleSmartShoppingPortalConfig();
    my $MockedShop = $self->_createShop({ 'PortalConfigs' => [$GooglePortalConfig] });

    my $SUT = $self->_createSUT($MockedShop,[$GoogleSmartShoppingPortal]);

    my $PortalConfigCreationParams = undef;
    lives_ok(
        sub {
            $PortalConfigCreationParams = $SUT->deleteSmartShoppingPortalConfig();
        },
        "$SubTestName - GetPortalConfigCreationParams can be called without errors."
    );

    ok(WasCalled($GooglePortalConfig, 'delete'), "$SubTestName - confirm delete was called on PortalConfig" )
}

sub test_DeleteProductsInGooglePortalConfig_success {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingPortal = $self->_createPortal({'NameOrAlias' => GOOGLE_SMART_SHOPPING_PORTAL_ALIAS, 'PortalSiteLocaleID' => 'de_CH' });
    my $GooglePortalConfig = $self->_createGoogleSmartShoppingPortalConfig();
    my $MockedShop = $self->_createShop({ 'PortalConfigs' => [ $GooglePortalConfig ], 'IsShopFolder' => 1  });
    my $SUT = $self->_createSUT($MockedShop,[$GoogleSmartShoppingPortal]);

    my $aProductAliases = ['32-A Glove that fits', '44 A bag of chips'];
    lives_ok(
        sub {
            $SUT->deleteProductsInPortalConfig($aProductAliases);
        },
        "$SubTestName - deleteProductsInPortalConfig can be called without errors."
    );

    ok(WasCalled($GooglePortalConfig, 'removeProducts'), "$SubTestName - confirm removeProducts was called on PortalConfig" )
}

sub test_getProductAliasByGoogleID_success {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $SUT = $self->_createSUT();
    my $aProductGoogleIDs = ["de:DE:32-Stone-Butch-Blues_de_DE", "de:DE:32-Souls of Black Folk_de_DE"];
    my $AliasesFromGoogleID = [];
    lives_ok(
        sub {
            $AliasesFromGoogleID = $SUT->getProductAliasByGoogleID($aProductGoogleIDs);
        },
        "$SubTestName - deleteProductsInPortalConfig can be called without errors."
    );
    my $ExpectedAliases = ["32-Stone-Butch-Blues", "32-Souls of Black Folk" ];

    is_deeply($AliasesFromGoogleID,$ExpectedAliases, "$SubTestName - Check if Aliases were correctly restored from IDs");
}

sub _createSUT {
    my $self = shift;
    my ($Shop, $aPortals) = @_;
    my $hServiceParams = { 'Shop' => $Shop // undef };
    my $MockifySUT = Test::Mockify::Sut->new('DE_EPAGES::GoogleSmartShopping::API::GooglePortalService', [$hServiceParams]);

    $MockifySUT->mockImported('DE_EPAGES::GoogleSmartShopping::API::Tools', 'LogGoogleSmartShopping')
        ->whenAny()
        ->thenReturnUndef();

    $MockifySUT->mockImported('DE_EPAGES::Core::API::Log', 'LogError')
        ->whenAny()
        ->thenReturnUndef();

    $MockifySUT->mockImported('DE_EPAGES::Object::API::Factory', 'LoadRootObject')
        ->whenAny()
        ->thenReturn($self->_createSystem($aPortals // undef));

    return $MockifySUT->getMockObject();
}

sub _createGoogleSmartShoppingPortalConfig {
    my $self = shift;

    return  $self->_createMockedPortalConfig ({'Alias' => 'GoogleSmartShopping', 'Package' => 'DE_EPAGES::GoogleSmartShopping::API::Object::GoogleSmartShoppingPortalConfig'})
}

sub _createIdealoConfig {
    my $self = shift;

    return  $self->_createMockedPortalConfig ({'Alias' => 'Idealo', 'Package' => 'DE_EPAGES::Idealo::API::Object::IdealoConfig'})
}

sub _createMockedPortalConfig {
    my $self = shift;
    my ($hConfigAttributes) = @_;

    my $hAttributes = {
        Alias => $hConfigAttributes->{'Alias'} // undef,
        ID    => '23456',
    };

    my $MockifyConfig = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        $hConfigAttributes->{'Package'},
        $hAttributes
    );

    $MockifyConfig->mock('delete', undef);
    $MockifyConfig->addMockWithReturnValueAndParameterCheck('removeProducts', undef, [ 'arrayref' ]);

    return $MockifyConfig->getMockObject();
}

sub _createShop {
    my $self = shift;
    my ($hParams) = @_;
    my $aPortalConfigs = $hParams->{'PortalConfigs'};
    my $hAttributes = {

        'ID'            => undef,
        'LocaleID'      => LOCALE_ID,
        'LanguageID'    => 1,
        'NameOrAlias'   => 'ShopNameOrAlias',
        'PortalConfigs' => $aPortalConfigs
    };

    my $MockifyShop = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', $hAttributes);

    if (defined $hParams->{'IsShopFolder'}) {
        $MockifyShop->mock('folder', sub {return $self->_createFolderObject()});
    }
    else {
        $MockifyShop->mock('folder', sub {return $self->_createPortalConfigsFolder()});
    }


    return $MockifyShop->getMockObject();
}

sub _createPortalConfigsFolder {
    my $self = shift;

    my $hAttributes = {
        'Alias' => 'PortalConfigs',
    };

    my $MockifySystem = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::ProductPortal::API::Object::PortalConfig', $hAttributes
    );

    return $MockifySystem->getMockObject();
}

sub _createFolderObject {
    my $self = shift;

    my $hAttributes = {
        Alias => 'PortalConfigs'
    };
    my $Mockify =  Test::Mockify->new('DE_EPAGES::Object::API::Object::Object', [$hAttributes]);

    $Mockify->mock('existsChild')
        ->when(String())
        ->thenReturn(1);

    $Mockify->mock('child')
        ->when(String())
        ->thenReturn($self->_createProduct());

    return $Mockify->getMockObject();
}

sub _createProduct {
    my $self = shift;
    my $hParameterList = {
        Alias => '32-Bica',
        ID    => 12345,
    };

    my $MockProduct = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Product::API::Object::Product',
        $hParameterList
    );

    return $MockProduct->getMockObject();
}

sub _createSystem {
    my $self = shift;
    my ($aPortals) = @_;

    my $hAttributes = {
        'Portals' => $aPortals // undef,
    };

    my $MockifySystem = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Object::API::Object::Site', $hAttributes
    );

    return $MockifySystem->getMockObject();
}

sub _createPortal {
    my $self = shift;
    my ($hParams) = @_;

    my $hAttributes = {
        'NameOrAlias'       => $hParams->{'NameOrAlias'} // undef,
        'PortalSites'       => [ $self->_createPortalSites($hParams->{'PortalSiteLocaleID'} // undef) ],
        'PortalConfigClass' => PORTAL_CONFIG_CLASS,
    };

    my $MockifyPortal = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::ProductPortal::API::Object::Portal', $hAttributes
    );

    return $MockifyPortal->getMockObject();
}

sub _createPortalSites {
    my $self = shift;
    my ($LocaleID) = @_;
    my $hAttributes = {
        'LocaleID' => $LocaleID // LOCALE_ID,
    };

    my $MockifyPortalSite = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::ProductPortal::API::Object::PortalSite', $hAttributes
    );

    return $MockifyPortalSite->getMockObject();
}

__PACKAGE__->runScript();

1;
