#========================================================================================
# package      DE_EPAGES::GoogleSmartShopping::t::unit::PagerMerchantCenterProducts
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for the GoogleSmartShopping ProductFeed tab
#========================================================================================
package DE_EPAGES::GoogleSmartShopping::t::unit::PagerMerchantCenterProducts;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::MockifyServlet;

use Test::Mockify;
use Test::Mockify::Matcher qw ( String HashRef Any Object );
use Test::Mockify::Sut;
use Test::Mockify::Verify qw ( WasCalled GetParametersFromMockifyCall );

use Test::More;
use Test::Exception;

use strict;

sub test {
    my $self = shift;

    $self->test_Pager_Execute_Success_NoProducts();
    $self->test_Pager_Execute_Success_WithProducts();
    $self->test_Pager_Execute_Success_EmptyProducts();

    done_testing();
}


sub test_Pager_Execute_Success_NoProducts {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedShop = $self->_createShop();
    my $MockedPortalConfig = $self->_createPortalConfig({ MockedShop => $MockedShop});
    my $GoogleApiService = $self->_createGoogleApiService({ 'ProductsStatuses' => {} });

    # when Servlet base class calls ExistsClassByAlias, call without Servlet
    my $hParams = {
        'Object'           => $MockedPortalConfig,
        'ObjectID'         => '12345',
        'GoogleApiService' => $GoogleApiService,
    };

    my $SUT = $self->_createSUT($hParams);

    is_deeply($SUT->items(),[], "$SubTestName - Verify pager items.");
    is($SUT->countItems(),0, "$SubTestName - Verify pager count items.");

    $SUT->execute($GoogleApiService);

    ok(WasCalled($GoogleApiService, 'productsStatuses'),
        $SubTestName . ' - Verify that service method productsStatuses was called.');

}

sub test_Pager_Execute_Success_WithProducts {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedShop = $self->_createShop();
    my $MockedPortalConfig = $self->_createPortalConfig({ MockedShop => $MockedShop});

    my $merchantCenterItems = [{
        'issues' => [],
        'lastUpdate' => '2022-12-02T12:10:44Z',
        'productId' => 'online:es:ES:3796',
        'status' => 'approved'
      },
      {
        'issues' => [],
        'lastUpdate' => '2022-11-30T17:20:23Z',
        'productId' => 'online:es:ES:_champu_pc_400ml',
        'status' => 'approved'
      },
      {
        'issues' => [],
        'lastUpdate' => '2022-11-30T17:20:23Z',
        'productId' => 'online:es:ES:_champu_pc_400ml_de_DE',
        'status' => 'approved'
      }
    ];

    my $GoogleApiService = $self->_createGoogleApiService({ 'ProductsStatuses' => {'items' => $merchantCenterItems}});

    # when Servlet base class calls ExistsClassByAlias, call without Servlet
    my $hParams = {
        'Object'           => $MockedPortalConfig,
        'ObjectID'         => '12345',
        'GoogleApiService' => $GoogleApiService,
    };

    my $SUT = $self->_createSUT($hParams);

    $SUT->set({
         'MCItems' => $merchantCenterItems,
         'MCItemCount' => 3,
    });

    is_deeply($SUT->items(),$merchantCenterItems, "$SubTestName - Verify pager items.");

    is($SUT->countItems(),3, "$SubTestName - Verify pager count items.");

    $SUT->execute($GoogleApiService);

    ok(WasCalled($GoogleApiService, 'productsStatuses'),
        $SubTestName . ' - Verify that service method productsStatuses was called.');

    ok(WasCalled($SUT, '_getProductByAlias'),
        $SubTestName . ' - Verify that method _getProductByAlias was called.');

    my $aGetProductByAliasParams = GetParametersFromMockifyCall($SUT, '_getProductByAlias');
    is( $aGetProductByAliasParams->[1], '_champu_pc_400ml', "$SubTestName - Verify $aGetProductByAliasParams Url parameter is correct." );

}

sub test_Pager_Execute_Success_EmptyProducts {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedShop = $self->_createShop();
    my $MockedPortalConfig = $self->_createPortalConfig({ MockedShop => $MockedShop});

    my $merchantCenterItems = [{
        'issues' => [],
        'lastUpdate' => '2022-12-02T12:10:44Z',
        'productId' => 'online:es:ES:wer_we_rere',
        'status' => 'approved'
      },
      {
        'issues' => [],
        'lastUpdate' => '2022-11-30T17:20:23Z',
        'productId' => 'online:es:ES:1018',
        'status' => 'approved'
      },
      {
        'issues' => [],
        'lastUpdate' => '2022-11-30T17:20:23Z',
        'productId' => 'online:es:ES:',
        'status' => 'approved'
      }
    ];

    my $GoogleApiService = $self->_createGoogleApiService({ 'ProductsStatuses' => {'items' => $merchantCenterItems}});

    # when Servlet base class calls ExistsClassByAlias, call without Servlet
    my $hParams = {
        'Object'           => $MockedPortalConfig,
        'ObjectID'         => '12345',
        'GoogleApiService' => $GoogleApiService,
    };

    my $SUT = $self->_createSUT($hParams);

    $SUT->set({
         'MCItems' => $merchantCenterItems,
         'MCItemCount' => 3,
    });

    lives_ok( sub { $SUT->items(); }, $SubTestName . ' - Searching for existing products does not crash.' );

    is_deeply($SUT->items(),$merchantCenterItems, "$SubTestName - Verify pager items.");

    is($SUT->countItems(),3, "$SubTestName - Verify pager count items.");

    $SUT->execute($GoogleApiService);

    ok(WasCalled($GoogleApiService, 'productsStatuses'),
        $SubTestName . ' - Verify that service method productsStatuses was called.');

    ok(!WasCalled($SUT, '_getProductByAlias'),
        $SubTestName . ' - Verify that method _getProductByAlias was NOT called.');

}

#----------------------------------------------------------------------------------------
# Mockery
#----------------------------------------------------------------------------------------
sub _createSUT {
    my $self = shift;
    my ($hParams ) = @_;

    my $MockifySut = Test::Mockify::Sut->new('DE_EPAGES::GoogleSmartShopping::API::Object::PagerMerchantCenterProducts',[$hParams]);

    $MockifySut->mockImported('DE_EPAGES::GoogleSmartShopping::API::Tools', 'LogGoogleSmartShopping')
        ->whenAny()
        ->thenReturnUndef();

    if (defined $hParams->{'GoogleApiService'}) {
        $MockifySut->mockStatic('DE_EPAGES::GoogleSmartShopping::API::GoogleApiService::new')
            ->when(String(), HashRef())
            ->thenReturn($hParams->{'GoogleApiService'});
    }

    $MockifySut->spy('_getProductByAlias')->when(Object(), String());

    return $MockifySut->getMockObject();
}

sub _createPortalConfig {
    my $self = shift;
    my ($hParams) = @_;

    my $hAttributes = {};

    my $MockifyPortalConfig = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::GoogleSmartShopping::API::Object::GoogleSmartShoppingPortalConfig', $hAttributes
    );

    $MockifyPortalConfig->addMockWithReturnValue('getSite', $hParams->{'MockedShop'});

    return $MockifyPortalConfig->getMockObject();
}

sub _createShop {
    my $self = shift;

    my $MockifyShop = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', {});
    $MockifyShop->mock('folder', sub {return $self->_createFolderObject()});

    return $MockifyShop->getMockObject();
}

sub _createFolderObject {
    my $self = shift;

    my $hAttributes = {
        Alias => 'PortalConfigs'
    };
    my $Mockify =  Test::Mockify->new('DE_EPAGES::Object::API::Object::Object', [$hAttributes]);

    $Mockify->mock('existsChild')
        ->when(Any())
        ->thenReturn(0);

    return $Mockify->getMockObject();
}

sub _createGoogleApiService {
    my $self = shift;
    my ($hServiceResponse) = @_;

    my $Mockify = Test::Mockify->new('DE_EPAGES::GoogleSmartShopping::API::GoogleApiService');

    $Mockify->mock('productsStatuses')->when()->thenReturn($hServiceResponse->{'ProductsStatuses'} // {});

    return $Mockify->getMockObject();
}

__PACKAGE__->runScript();
