#========================================================================================
# package      DE_EPAGES::Object::API::Cartridge
# base         DE_EPAGES::Object::API::ObjectInstaller
# state        public
#----------------------------------------------------------------------------------------
# description  This is the main cartridge class for install/patch/uninstall.
#========================================================================================
package DE_EPAGES::Object::API::Cartridge;
use base DE_EPAGES::Object::API::ObjectInstaller;

use strict;
use DE_EPAGES::Core::API::Error qw( Error );
use DE_EPAGES::Core::API::EpagesConfig qw( EpagesConfig );
use DE_EPAGES::Core::API::Log qw ( GetLog );
use DE_EPAGES::Core::API::File qw ( RunOnGlob CopyDirectory );
use DE_EPAGES::Core::API::Net qw( localhostName );
use DE_EPAGES::Core::API::PerlTools qw( trim );
use DE_EPAGES::Object::API::Factory qw( LoadClassByAlias LoadRootObject Transaction );
use DE_EPAGES::Object::API::Language qw( GetPKeyLanguageByCode );
use DE_EPAGES::Object::API::Object::Class;
use DE_EPAGES::Object::API::Table::Currency qw( UpdateCurrency );
use DE_EPAGES::Object::API::Table::Language qw( GetInfoLanguagesByCode2 UpdateLanguage );
use DE_EPAGES::Object::API::XML::AttributeImportHandler qw( ImportTranslation );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (ImportObjects);

#========================================================================================
# function     new
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer = DE_EPAGES::Object::API::Cartridge->new(%options);
#----------------------------------------------------------------------------------------
# description  Creates a new installer object of DE_EPAGES::Object.
#----------------------------------------------------------------------------------------
# input        %options | options for un/install
#               <ul>
#                 <li>IsRecursive - recursive un/installation of required or dependent
#                     cartridges
#               </ul>
#               | hash
# return       $Installer | cartridge installer | object
#========================================================================================
sub new {
    my ($class, %options) = @_;

    my $self = __PACKAGE__->SUPER::new(
        %options,
        'CartridgeDirectory' => 'DE_EPAGES/Object',
        'Version'            => '7.79.0',
        'Patches'            => ['6.15.1','6.15.2','6.15.3','6.15.4','6.16.0','6.16.1','6.16.2','6.16.3','6.17.0','6.17.1','6.17.2','6.17.3','6.17.4','6.17.5','6.17.6','6.17.7','6.17.8','6.17.9','6.17.9.1','6.17.9.2','6.17.9.3','6.17.10','6.17.11','6.17.12','6.17.12.1','6.17.12.2','6.17.13','6.17.13.1','6.17.13.2','6.17.14','6.17.14.1','6.17.15','6.17.15.1','6.17.16','6.17.17','6.17.17.1','6.17.18','6.17.19','6.17.20','6.17.20.1','6.17.21','6.17.21.1','6.17.22','6.17.23','6.17.24','6.17.24.1','6.17.25','6.17.26','6.17.26.1','6.17.27','6.17.27.1','6.17.28','6.17.28.1','6.17.29','6.17.29.1','6.17.30','6.17.31','6.17.32','6.17.33','6.17.33.1','6.17.34','6.17.34.1','6.17.35','6.17.36','6.17.36.1','6.17.37','6.17.38','6.17.38.1','6.17.39','6.17.39.1','6.17.40','6.17.40.1','6.17.41','6.17.41.1','6.17.42','6.17.43','6.17.43.1','6.17.44','6.17.45','6.17.45.1','6.17.46','6.17.47','6.17.47.1','6.17.48','6.17.48.1','6.17.49','6.17.49.1','6.17.50','6.17.51','6.17.51.1','6.17.52','6.17.53','7.0.0','7.1.0','7.2.0','7.3.0','7.4.0','7.5.0','7.6.0','7.7.0','7.8.0','7.9.0','7.10.0','7.11.0','7.12.0','7.13.0','7.14.0','7.15.0','7.16.0','7.17.0','7.18.0','7.19.0','7.20.0','7.21.0','7.22.0','7.23.0','7.24.0','7.25.0','7.26.0','7.27.0','7.28.0','7.29.0','7.30.0','7.31.0','7.32.0','7.33.0','7.34.0','7.35.0','7.36.0','7.37.0','7.38.0','7.39.0','7.40.0','7.41.0','7.42.0','7.43.0','7.44.0','7.45.0','7.46.0','7.47.0','7.48.0','7.49.0','7.50.0','7.51.0','7.52.0','7.53.0','7.54.0','7.55.0','7.56.0','7.57.0','7.58.0','7.59.0','7.60.0','7.61.0','7.62.0','7.63.0','7.64.0','7.65.0','7.66.0','7.67.0','7.68.0','7.69.0','7.70.0','7.71.0','7.72.0','7.73.0','7.74.0','7.75.0','7.76.0','7.77.0','7.78.0'],
    );
    return bless $self, $class;
}

#========================================================================================
# function     install
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->install;
#----------------------------------------------------------------------------------------
# description  Installs the cartridge.
#========================================================================================
sub install {
    my $self = shift;

    $self->startInstall;

    # note: ObjectInstaller::install() cannot be used here, because it requires initStructure()
    $self->DE_EPAGES::Cartridge::API::CartridgeInstaller::install;

    # insert the root object
    DE_EPAGES::Object::API::Object::Class->initStructure;

    # import languages etc
    Transaction( sub {
        GetLog->debug(sprintf('import standards %s into store %s.', $self->cartridgePackage, $self->storeName));
        ImportObjects( $self->xmlDirectory . '/Languages.xml');
        ImportObjects( $self->xmlDirectory . '/Currencies_4217.xml');

        GetLog->debug(sprintf('install objects %s on store %s.', $self->cartridgePackage, $self->storeName));
        my $XMLDir = $self->xmlDirectory;
        RunOnGlob($XMLDir, 'Attributes*.xml', sub { ImportObjects( $_ ) });

        ImportObjects( $XMLDir . '/Countries_3166.xml');
        RunOnGlob($XMLDir, 'CountrySubdivision*.xml', sub { ImportObjects( $_ ) });

        ImportObjects( $XMLDir . '/Locales.xml');
        ImportObjects( $XMLDir . '/System.xml', '/');
        ImportObjects( $XMLDir . '/UnitsOfMeasurement.xml', '/');

        $self->_activateLocales;

        ImportTranslation( "$XMLDir/Translation.Attributes" );
        ImportTranslation( "$XMLDir/Translation.UnitsOfMeasurement" );
    });

    # update the web server URL
    my $System = LoadRootObject();
    my $SystemDomainName = EpagesConfig()->get( 'DE_EPAGES::Object', 'SystemDomainName',  localhostName() );
    $System->set({ DomainName => $SystemDomainName });
    $System->set({ WebServerScriptNamePart => $self->storeName });

    $self->_copyWebRootIcons;

    $self->finishInstall;
    return;
}

#========================================================================================
# function     _activateLocales
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->_activateLocales();
#----------------------------------------------------------------------------------------
# description  Activates languages, locales and currenies for a new database.
#               The defaults can be overloaded by setting environment variables, e.g.:
#               EPAGES_LANGUAGES=de en-US
#               EPAGES_LOCALES=de_DE de_AT en_GB en_US
#               EPAGES_CURRENCIES=EUR GBP USD
#========================================================================================
sub _activateLocales {
    my $self = shift;

    my $Languages = $ENV{EPAGES_LANGUAGES} || 'en de';
    my @LanguageIdentifiers = map { trim($_) } split /\s+/, $Languages;
    my @LanguageCodes = map { /^(\w+)-\w+$/ ? $1 : $_ } @LanguageIdentifiers;
    my @LanguageIDs = map { GetPKeyLanguageByCode( $_ ) } @LanguageCodes;
    Error('NoLanguages') if 0 == scalar @LanguageIDs;

    # first locale becomes the system default
    my $Locales = $ENV{EPAGES_LOCALES} || 'en_GB en_US en_AU en_IE en_CA en_NZ en_CH de_DE de_CH de_AT de_LU de_BE';
    my @LocaleIDs = map { trim($_) } split /\s+/, $Locales;
    Error('NoLocales') if 0 == scalar @LocaleIDs;

    my $Currencies = $ENV{EPAGES_CURRENCIES} || 'USD GBP EUR JPY SEK AUD CHF CAD NZD';
    my @CurrencyIDs = map { trim($_) } split /\s+/, $Currencies;
    Error('NoCurrencies') if 0 == scalar @CurrencyIDs;

    # activate languages
    UpdateLanguage( $_, { IsUsed => 1 } ) foreach @LanguageIDs;

    # switch dialects by importing a file like "Languages.en-US.xml"
    my @Dialects = grep { /^(\w+)-(\w+)/ } @LanguageIdentifiers;
    ImportObjects( $self->xmlDirectory . "/Languages.$_.xml" ) foreach @Dialects;

    # activate locales
    my $LocaleClass = LoadClassByAlias('Locale');
    $LocaleClass->loadObject($_)->set({ IsUsed => 1 }) foreach @LocaleIDs;

    # activate currencies
    UpdateCurrency( $_, { IsUsed => 1 } ) foreach @CurrencyIDs;

    # deactivate german (de)
    my $LanguageID_DE = GetPKeyLanguageByCode('de');
    unless( $LanguageID_DE ~~ @LanguageIDs ) {
        UpdateLanguage( $LanguageID_DE, { IsUsed => 0 } );
    }

    # decativate german (de_DE)
    unless( 'de_DE' ~~ @LocaleIDs ) {
        $LocaleClass->loadObject('de_DE')->set({ IsUsed => 0 });
    }

    my $System = LoadRootObject();
    # set system default locale
    $System->set({ LocaleID => $LocaleIDs[0] });
    # set system timezone if defined ENV{EPAGES_TIMEZONE}
    if (defined $ENV{EPAGES_TIMEZONE} and DateTime::TimeZone->is_valid_name($ENV{EPAGES_TIMEZONE})) {
        $System->set({'TimeZone' => $ENV{EPAGES_TIMEZONE} });
    }
    return;
}

#========================================================================================
# function     _copyWebRootIcons
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->_copyWebRootIcons();
#----------------------------------------------------------------------------------------
# description  copy images in Data/Public/SF/{Countries,Locales} also to WebRoot/Icons
#========================================================================================
sub _copyWebRootIcons {
    my $self = shift;

    # EPG-27542: copy images in Data/Public/SF/{Countries,Locales} also to WebRoot/Icons
    # overwrites the method from Core::API::FileInstaller
    my $SourceDir = join ('/', $ENV{EPAGES_CARTRIDGES}, $self->cartridgeDirectory, 'Data/Public/SF');
    my $TargetDir = "$ENV{EPAGES_WEBROOT}/Icons";

    CopyDirectory("$SourceDir/$_", "$TargetDir/$_") foreach(qw(Countries Locales));
}


#========================================================================================
# function     uninstall
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->uninstall;
#----------------------------------------------------------------------------------------
# description  Uninstalls the cartridge.
#========================================================================================
sub uninstall {
    my $self = shift;

    $self->startUninstall;
    $self->SUPER::uninstall;
    $self->finishUninstall;
    return;
}

#========================================================================================
# function     patchDBStructure_6_15_1
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->patchDBStructure_6_15_1;
#----------------------------------------------------------------------------------------
# description  Patches cartridge db structure in active store (database connection)
#               from version '6.15.1' to version '6.15.2'.
#----------------------------------------------------------------------------------------
# return       $Version | version of cartridge after patch function, '6.15.2' | string
#========================================================================================
sub patchDBStructure_6_15_1 {
    my $self = shift;

    my $dbi = $self->getCurrentDBHandle();

    if($dbi->driverName() eq 'mysql'){

        #changes for file /Database/mysql/Tables/country.sql
        $self->logInfo('change type of column country.subdivsmandatory');

        $dbi->do(<<'END_SQL',[0]);
UPDATE country
SET subdivsmandatory=?
WHERE subdivsmandatory IS NULL
END_SQL

        $dbi->modifyTableColumn('country', 'subdivsmandatory bool not null');

    }

    $self->importPatchDir('Patch_6_15_1', 'PatchDBStructure');

    return '6.15.2';
}

sub patch_6_16_0 {
    my $self = shift;

    $self->importPatchDir('Patch_6_16_0', 'Patch');

    $self->logInfo('repair objects where the siteid does not match the parentid');
    $self->fixSiteID_6_16_0( $self->getCurrentDBHandle );

    return '6.16.1';
}

sub fixSiteID_6_16_0 { # see EPG-29498
    my $self = shift;
    my ($dbi) = @_;

    $dbi->do('CREATE TEMPORARY TABLE sysobject( objectid INT PRIMARY KEY, level INT NOT NULL )');
    $dbi->do('CREATE TEMPORARY TABLE parentsysobject( objectid INT PRIMARY KEY )');
    $dbi->do('CREATE TEMPORARY TABLE childsysobject( objectid INT PRIMARY KEY )');
    $dbi->do('CREATE TEMPORARY TABLE invalidsysobject( objectid INT PRIMARY KEY, siteid INT NOT NULL )');
    $dbi->do('DROP PROCEDURE IF EXISTS findInvalidSysObjects');
    $dbi->do(<<'END_SQL');
CREATE PROCEDURE findInvalidSysObjects()
BEGIN

    DECLARE lev INT DEFAULT 1;
    DECLARE children INT DEFAULT 0;

    DELETE FROM parentsysobject;

    INSERT into parentsysobject( objectid )
    SELECT 1;

    INSERT into sysobject( objectid, level )
    SELECT objectid, lev
    FROM parentsysobject;

    REPEAT

        SELECT lev + 1 INTO lev;

        INSERT into childsysobject( objectid )
        SELECT objectid
        FROM object
        WHERE parentid IN (
            SELECT objectid
            FROM parentsysobject
        )
          AND objectid <> siteid;

        SELECT ROW_COUNT() INTO children;

        INSERT into sysobject( objectid, level )
        SELECT objectid, lev
        FROM childsysobject;

        DELETE FROM parentsysobject;

        INSERT into parentsysobject( objectid )
        SELECT objectid
        FROM childsysobject;

        DELETE FROM childsysobject;

    UNTIL children = 0
        END REPEAT;

    SELECT *
    FROM object
    WHERE siteid = 1
      AND objectid NOT IN(
        SELECT objectid
        FROM sysobject
    );
END
END_SQL

    $dbi->do('CALL findInvalidSysObjects()');
    $dbi->do(<<'END_SQL');
INSERT INTO invalidsysobject( objectid, siteid )
SELECT objectid, parentid
FROM object
WHERE siteid = 1
  AND objectid NOT IN(
    SELECT objectid
    FROM sysobject
)
END_SQL

    $dbi->do(<<'END_SQL') for( 1..3 ); # go up 3 layers; not recursively, because there are cycles */
UPDATE invalidsysobject i
INNER JOIN object o ON i.siteid = o.objectid
SET i.siteid = o.parentid
WHERE o.objectid <> o.siteid;
END_SQL

    # fix cycles
    $dbi->do('DELETE FROM invalidsysobject WHERE siteid NOT IN( SELECT siteid FROM site )');

    # fix SiteID colum
    $dbi->do(<<'END_SQL');
UPDATE object o
INNER JOIN invalidsysobject i ON i.objectid = o.objectid
SET o.siteid = i.siteid;
END_SQL

    $dbi->dropProcedure('findInvalidSysObjects');
    $dbi->dropTable('invalidsysobject');
    $dbi->dropTable('childsysobject');
    $dbi->dropTable('parentsysobject');
    $dbi->dropTable('sysobject');
}

sub patch_6_16_3 {
    my $self = shift;

    $self->importPatchDir('Patch_6_16_3', 'Patch');

    # See EPG-30953, EPG-31529
    if( GetInfoLanguagesByCode2('de')->[0]->{'IsUsed'} ) {
        $self->logInfo('activate system locale de_BE');

        my $Locale = LoadClassByAlias('Locale')->loadObject('de_BE');
        $Locale->set({ IsUsed => 1 });
    }

    return '6.17.0';
}

sub patch_6_17_3 {
    my $self = shift;

    $self->importPatchDir('Patch_6_17_3', 'Patch');

    #set es_MX is used
    if( GetInfoLanguagesByCode2('es')->[0]->{'IsUsed'} ) {
        $self->logInfo('activate system locale es_MX');

        my $Locale = LoadClassByAlias('Locale')->loadObject('es_MX');
        $Locale->set({ IsUsed => 1 });
    }

    return '6.17.4';
}

sub patch_6_17_28_1 {
    my $self = shift;

    $self->importPatchDir('Patch_6_17_28_1', 'Patch');

    #set es_EC is used (AD-2094)
    if( GetInfoLanguagesByCode2('es')->[0]->{'IsUsed'} ) {
        $self->logInfo('activate system locale es_EC');

        my $Locale = LoadClassByAlias('Locale')->loadObject('es_EC');
        $Locale->set({ IsUsed => 1 });
    }

    return '6.17.29';
}

#========================================================================================
# function     patchDBStructure_6_17_5
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->patchDBStructure_6_17_5;
#----------------------------------------------------------------------------------------
# description  Patches cartridge db structure in active store (database connection)
#               from version '6.17.5' to version '6.17.6'.
#----------------------------------------------------------------------------------------
# return       $Version | version of cartridge after patch function, '6.17.6' | string
#========================================================================================
sub patchDBStructure_6_17_5 {
    my $self = shift;

    my $dbi = $self->getCurrentDBHandle();

    unless($dbi->existsTable('lastupdateprofile')){
        # Only if table not exist
        $dbi->do(<<'END_SQL');
CREATE TABLE lastupdateprofile (
     profileid            integer  not null
,    profileclassid       integer  not null

,   constraint pk_lastupdateprofile             primary key (profileid)
,   constraint fk_lastupdateprofile_profileid   foreign key (profileid)
        references object                           (objectid)
,   constraint fk_lastupdateprofile_profileclassid foreign key (profileclassid)
        references class                            (classid)
)
END_SQL

        $dbi->do(<<'END_SQL');
CREATE INDEX i_lastupdateprofile_class ON lastupdateprofile (profileclassid)
END_SQL
    }

    unless($dbi->existsTable('lastupdate')){
        # Only if table not exist
        $dbi->do(<<'END_SQL');
CREATE TABLE lastupdate (
     objectid             integer  not null
,    profileid            integer  not null
,    siteid               integer  not null
,    lastupdatedon        datetime  not null

,   constraint pk_lastupdate                    primary key (objectid,profileid)
,   constraint fk_lastupdate_objectid           foreign key (objectid)
        references object                           (objectid)
,   constraint fk_lastupdate_siteid             foreign key (siteid)
        references site                             (siteid)
,   constraint fk_lastupdate_profileid          foreign key (profileid)
        references lastupdateprofile                (profileid)
)
END_SQL
        $dbi->do(<<'END_SQL');
CREATE INDEX i_lastupdate ON lastupdate (siteid, profileid, lastupdatedon)
END_SQL

        $dbi->do(<<'END_SQL');
CREATE INDEX i_lastupdate_profile ON lastupdate (profileid)
END_SQL
    }

    unless($dbi->existsTable('objectdelete')){
        # Only if table not exist
        $dbi->do(<<'END_SQL');
CREATE TABLE objectdelete (
     guid                 varchar(36)  not null
,    siteid               integer  not null
,    classid              integer  not null
,    pathfromsite         varchar(1000)  not null
,    deletedon            datetime  not null

,   constraint pk_objectdelete                  primary key (guid)
,   constraint fk_objectdelete_classid          foreign key (classid)
        references class                            (classid)
,   constraint fk_objectdelete_siteid           foreign key (siteid)
        references site                             (siteid)
)
END_SQL

        $dbi->do(<<'END_SQL');
CREATE INDEX i_objectdelete ON objectdelete (siteid, classid, deletedon)
END_SQL

        $dbi->do(<<'END_SQL');
CREATE INDEX i_objectdelete_class ON objectdelete (classid)
END_SQL

    }

    return '6.17.6';
}

sub patch_7_33_0 {
    my $self = shift;

    $self->importPatchDir('Patch_7_33_0', 'Patch');

    $self->_copyWebRootIcons;
    return '7.34.0';
}

sub patch_7_42_0 {
    my $self = shift;

    $self->importPatchDir('Patch_7_42_0', 'Patch');

    # See AD-9649 - add en_CH, copy icons
    if( GetInfoLanguagesByCode2('en')->[0]->{'IsUsed'} ) {
        $self->logInfo('activate system locale en_CH');

        my $Locale = LoadClassByAlias('Locale')->loadObject('en_CH');
        $Locale->set({ IsUsed => 1 });
    }
    $self->_copyWebRootIcons;
    return '7.43.0';
}

1;
