#========================================================================================
# package      DE_EPAGES::Product::API::Shop
# state        public
#----------------------------------------------------------------------------------------
# description  This package provides shop functions of Customer.
#========================================================================================
package DE_EPAGES::Product::API::Shop;
use base Exporter;
use strict;

our @EXPORT_OK = qw (
    GetProduct
    GetProductImageSizes
    UpdateAllProductsImageInformation
);

use DE_EPAGES::Object::API::Factory     qw ( LoadObject );
use DE_EPAGES::Product::API::Constants  qw ( PRODUCT_IMAGES_SIZES );
use DE_EPAGES::Product::DAL::Product    qw ( DAL_GetPKeyProductsByAlias );
use DE_EPAGES::Database::API::Connection qw( GetCurrentDBHandle );

#========================================================================================
# function     GetProduct
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Product = GetProduct($Shop, $Alias);
#----------------------------------------------------------------------------------------
# description  Checks if there is a product for a given product alias and returns it.
#               If 'UseMaster' is set then also super-products are returned, else the
#               default sub-product is returned.
#               If the alias doesn't clearly identify one product, e.g. in the case of
#               sub-products, the first sub-product with this alias which is found will
#               be returned. In case of entering the product number of a master product
#               the default sub product will be returned.
#               This function returns also termin resources (e.g. for cross-selling).
#----------------------------------------------------------------------------------------
# input        $Shop  | shop | object
# input        $Alias | alias string for product | string
# input        $UseMaster | (optional) if true then super-products are returned  | boolean
# return       $Product | product | object
#========================================================================================
sub GetProduct {
    my ($Shop, $Alias, $UseMaster) = @_;

    my $aProductIDs = DAL_GetPKeyProductsByAlias($Shop->id, $Alias);
    return undef unless scalar @$aProductIDs;

    my $Product = LoadObject(@$aProductIDs[0]);
    if (defined $UseMaster and $UseMaster == 1) {
        return $Product;
    } else {
        return defined $Product->get('DefaultSubProduct') ? $Product->get('DefaultSubProduct') : $Product;
    }
}

#========================================================================================
# function     GetProductImageSizes
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hSizes = GetProductImageSizes($Shop);
#----------------------------------------------------------------------------------------
# description  Returns the size values for the different image variations of the product
#               image.
#----------------------------------------------------------------------------------------
# input        $Shop  | shop | object
# return       $Sizes | image sizes | hash.ref
#========================================================================================
sub GetProductImageSizes {
    my ($Shop) = @_;
    return {
        %{PRODUCT_IMAGES_SIZES()},
        'l' => [ $Shop->get('ProductImageLargeWidth'), $Shop->get('ProductImageLargeHeight') ],
    };
}

#========================================================================================
# function     UpdateAllProductsImageInformation
# state        public
#----------------------------------------------------------------------------------------
# syntax       UpdateAllProductsImageInformation($Shop);
#----------------------------------------------------------------------------------------
# description  Updates the 'ImageInformations' attribute for all products in this shop
#----------------------------------------------------------------------------------------
# input        $Shop  | shop | object
# return        $NumUpdated | number of products updated | integer
#========================================================================================
sub UpdateAllProductsImageInformation {
    my ($Shop) = @_;

    return if ! $Shop->get('IsUnityRegistered');

    my $Dbi = GetCurrentDBHandle();

    my $aaResultSet = $Dbi->execute(<<'SQL'  ,[$Shop->id] );
SELECT p.productid
FROM product p
WHERE p.shopid = ?
ORDER BY p.productid ASC
SQL

    my $NumUpdated = 0;
    foreach my $aProductID (@$aaResultSet) {
        my $Product = $Shop->loadObject($aProductID->[0], 'Product');
        $Product->updateImageInformations(undef);
        $NumUpdated++;
    }

    return $NumUpdated;
}

1;
